import csv
from os.path import basename
import pickle
import wx


OK_BACKGROUND_COLOR = 'PALE GREEN'


def determine_wildcard(extension=None, file_type=None):
	"""
	Assemble a wildcard string of the form:
		[[file_type ](*.extension)|*.extension|]<all_files>
	"""

	all_files = 'All files|*'

	if extension is not None:
		if '|' in extension:
			raise ValueError(extension)

		if file_type is not None:
			if '|' in file_type:
				raise ValueError(file_type)

			wildcard = '{0} (*.{1})|*.{1}|{2}'.format(file_type, extension, all_files)
		else:
			wildcard = '(*.{0})|*.{0}|{1}'.format(extension, all_files)
	else:
		wildcard = all_files

	return wildcard


def load_pickled(parent, extension=None, file_type=None):
	"""
	Unpickle data from a file based on a file dialog.
	"""

	wildcard = determine_wildcard(extension, file_type)
	dlg = wx.FileDialog(parent=parent, message='Load...', wildcard=wildcard,
			style=wx.FD_OPEN)

	if dlg.ShowModal() == wx.ID_OK:
		path = dlg.GetPath()

		with open(path, 'rb') as f:
			try:
				return pickle.load(f)
			except Exception as e:
				# Wrap all problems.
				raise IOError('Could not load data.', e)

def save_pickled(parent, values, extension=None, file_type=None):
	"""
	Pickle data to a file based on a file dialog.
	"""

	wildcard = determine_wildcard(extension, file_type)
	dlg = wx.FileDialog(parent=parent, message='Save...',
			wildcard=wildcard, style=wx.FD_SAVE|wx.FD_OVERWRITE_PROMPT)

	if dlg.ShowModal() == wx.ID_OK:
		path = dlg.GetPath()

		# Automatically append extension if none given.
		if extension is not None and '.' not in path:
			path = '{0}.{1}'.format(path, extension)

		with open(path, 'wb') as f:
			try:
				pickle.dump(values, f, protocol=pickle.HIGHEST_PROTOCOL)
			except Exception as e:
				# Wrap all problems:
				raise IOError('Could not save data.', e)


def load_csv(parent, extension='csv', file_type='CSV'):
	"""
	Load data from a CSV file based on a file dialog.
	"""

	wildcard = determine_wildcard(extension, file_type)
	dlg = wx.FileDialog(parent=parent, message='Load...', wildcard=wildcard,
			style=wx.FD_OPEN)

	if dlg.ShowModal() == wx.ID_OK:
		path = dlg.GetPath()

		filename = basename(path)

		with open(path, 'rb') as f:
			try:
				result = list(csv.reader(f))
				try:
					has_header = len(result[0]) > 0
				except IndexError:
					has_header = False
				else:
					# Remove empty row.
					if not has_header:
						result = result[1:]

				return (has_header, result, filename)
			except Exception as e:
				# Wrap all problems.
				raise IOError('Could not load data.', e)

def save_csv(parent, values, headers=None, extension='csv', file_type='CSV'):
	"""
	Save data to a CSV file based on a file dialog.
	"""

	wildcard = determine_wildcard(extension, file_type)
	dlg = wx.FileDialog(parent=parent, message='Save...',
			wildcard=wildcard, style=wx.FD_SAVE|wx.FD_OVERWRITE_PROMPT)

	if dlg.ShowModal() == wx.ID_OK:
		path = dlg.GetPath()

		# Automatically append extension if none given.
		if extension is not None and '.' not in path:
			path = '{0}.{1}'.format(path, extension)

		with open(path, 'wb') as f:
			try:
				w = csv.writer(f)

				if headers is not None:
					w.writerow(headers)
				else:
					w.writerow([])

				w.writerows(values)
			except Exception as e:
				# Wrap all problems:
				raise IOError('Could not save data.', e)


class Dialog(wx.Dialog):
	"""
	Auto-destroying dialog.
	"""

	def __init__(self, parent, auto_destroy=True, *args, **kwargs):
		wx.Dialog.__init__(self, parent, *args, **kwargs)

		self.auto_destroy = auto_destroy

		self.Bind(wx.EVT_SHOW, self.OnShow)

	def OnShow(self, evt):
		"""
		Destroy the dialog when it disappears.
		"""

		if self.auto_destroy and not evt.Show:
			if not self.IsBeingDeleted():
				self.Destroy()


class MessageDialog(Dialog):
	"""
	A simple error message dialog.
	"""

	def __init__(self, parent, message, title='', unclosable=False, monospace=False,
			*args, **kwargs):
		kwargs['style'] = kwargs.get('style', wx.DEFAULT_DIALOG_STYLE) | wx.RESIZE_BORDER
		if unclosable:
			kwargs['style'] &= ~wx.CLOSE_BOX

		Dialog.__init__(self, parent=parent, title=title, *args, **kwargs)

		# Dialog.
		dialog_box = wx.BoxSizer(wx.VERTICAL)

		## Message.
		message_text = wx.StaticText(self, label=message)
		if monospace:
			font = message_text.Font
			message_text.Font = wx.Font(font.PointSize, wx.MODERN, font.Style, font.Weight)
		message_text.SetMinSize((450, 100))
		dialog_box.Add(message_text, proportion=1, flag=wx.EXPAND|wx.ALL, border=20)

		## OK button.
		if not unclosable:
			ok_button = wx.Button(self, wx.ID_OK)
			dialog_box.Add(ok_button, flag=wx.EXPAND)

		self.SetSizerAndFit(dialog_box)


class YesNoQuestionDialog(Dialog):
	"""
	A yes/no question dialog.
	"""

	def __init__(self, parent, prompt, yes_callback=None, no_callback=None, title='',
			*args, **kwargs):
		Dialog.__init__(self, parent=parent, title=title,
				style=wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER,
				*args, **kwargs)

		self.yes_callback = yes_callback
		self.no_callback = no_callback

		# Dialog.
		dialog_box = wx.BoxSizer(wx.VERTICAL)

		## Prompt.
		prompt_text = wx.StaticText(self, label=prompt)
		dialog_box.Add(prompt_text, proportion=1, flag=wx.EXPAND|wx.ALL, border=20)

		## Buttons.
		button_box = wx.BoxSizer(wx.HORIZONTAL)
		dialog_box.Add(button_box, flag=wx.CENTER)

		yes_button = wx.Button(self, wx.ID_YES)
		self.Bind(wx.EVT_BUTTON, self.OnYes, yes_button)
		button_box.Add(yes_button)

		no_button = wx.Button(self, wx.ID_NO)
		self.Bind(wx.EVT_BUTTON, self.OnNo, no_button)
		button_box.Add(no_button)

		self.SetSizerAndFit(dialog_box)

		self.Bind(wx.EVT_CLOSE, self.OnNo)

	def OnYes(self, evt=None):
		if self.yes_callback is not None:
			self.yes_callback()

		self.Destroy()

	def OnNo(self, evt=None):
		if self.no_callback is not None:
			self.no_callback()

		self.Destroy()



import logging
log = logging.getLogger(__name__)

import functools
import math
import numpy
from pubsub import pub
from threading import Lock
import time
import wx
from wx.lib.agw import floatspin

from spacq.interface.resources import AcquisitionThread
from spacq.interface.units import Quantity

from ....config.measurement import MeasurementConfigPanel
from ....tool.box import Dialog, MessageDialog

try:
	from ..two_dimensional import TwoDimensionalPlot
except ImportError as e:
	plot_available = False
	log.debug('Could not import TwoDimensionalPlot: {0}'.format(str(e)))
else:
	plot_available = True

"""
A historical live view plot for scalar values.
"""


class PlotSettings(object):
	"""
	Wrapper for all the settings configured via dialog.
	"""

	def __init__(self):
		self.enabled = plot_available
		self.num_points = 500
		self.delay = Quantity(0.2, 's')
		self.update_x = True
		self.time_value = 0
		self.time_mode = 0
		self.update_y = True
		self.y_scale = 0
		self.units_from = ''
		self.units_to = ''


class PlotSettingsDialog(Dialog):
	"""
	Set up the live view plot.
	"""

	def __init__(self, parent, ok_callback, *args, **kwargs):
		Dialog.__init__(self, parent=parent, title='Plot settings')

		self.ok_callback = ok_callback

		dialog_box = wx.BoxSizer(wx.VERTICAL)

		# Enabled.
		self.enabled_checkbox = wx.CheckBox(self, label='Enabled')
		if not plot_available:
			self.enabled_checkbox.Disable()
		dialog_box.Add(self.enabled_checkbox, flag=wx.ALL, border=5)

		# Capture.
		capture_static_box = wx.StaticBox(self, label='Capture')
		capture_box = wx.StaticBoxSizer(capture_static_box, wx.VERTICAL)
		capture_sizer = wx.FlexGridSizer(rows=2, cols=2, hgap=5)
		capture_box.Add(capture_sizer, flag=wx.CENTER)
		dialog_box.Add(capture_box, flag=wx.EXPAND|wx.ALL, border=5)

		## Number of points.
		capture_sizer.Add(wx.StaticText(self, label='Points:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.points_input = wx.SpinCtrl(self, min=2, max=1e4, initial=100)
		capture_sizer.Add(self.points_input, flag=wx.CENTER)

		## Delay.
		capture_sizer.Add(wx.StaticText(self, label='Delay (s):'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		# TODO: Input should be a time amount directly (eg. '200 ms').
		self.delay_input = floatspin.FloatSpin(self, min_val=0.2, max_val=1e4, increment=0.1, digits=2)
		capture_sizer.Add(self.delay_input, flag=wx.CENTER)

		# Axes.
		axes_static_box = wx.StaticBox(self, label='Axes')
		axes_box = wx.StaticBoxSizer(axes_static_box, wx.HORIZONTAL)
		dialog_box.Add(axes_box, flag=wx.EXPAND|wx.ALL, border=5)

		## x
		x_static_box = wx.StaticBox(self, label='x')
		x_box = wx.StaticBoxSizer(x_static_box, wx.VERTICAL)
		axes_box.Add(x_box, flag=wx.EXPAND)

		self.update_x_axis = wx.CheckBox(self, label='Autofit')
		x_box.Add(self.update_x_axis)

		### Value.
		self.time_value = wx.RadioBox(self, label='Value', choices=['Time', 'Points'])
		x_box.Add(self.time_value, flag=wx.EXPAND)

		### Mode.
		self.time_mode = wx.RadioBox(self, label='Mode', choices=['Relative', 'Absolute'])
		x_box.Add(self.time_mode, flag=wx.EXPAND)

		## y
		y_static_box = wx.StaticBox(self, label='y')
		y_box = wx.StaticBoxSizer(y_static_box, wx.VERTICAL)
		axes_box.Add(y_box, flag=wx.EXPAND)

		self.update_y_axis = wx.CheckBox(self, label='Autofit')
		y_box.Add(self.update_y_axis)

		### Conversion.
		conversion_static_box = wx.StaticBox(self, label='Conversion')
		conversion_box = wx.StaticBoxSizer(conversion_static_box, wx.VERTICAL)
		conversion_sizer = wx.FlexGridSizer(rows=1, cols=2, hgap=5)
		conversion_box.Add(conversion_sizer)
		y_box.Add(conversion_box)

		conversion_sizer.Add(wx.StaticText(self, label='Exp. scale:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.y_scale = floatspin.FloatSpin(self, min_val=-100, max_val=100, increment=1, digits=2)
		conversion_sizer.Add(self.y_scale)

		#### Units.
		units_static_box = wx.StaticBox(self, label='Units')
		units_box = wx.StaticBoxSizer(units_static_box, wx.VERTICAL)
		units_sizer = wx.FlexGridSizer(rows=2, cols=2, hgap=5)
		units_box.Add(units_sizer)
		conversion_box.Add(units_box, flag=wx.EXPAND)

		units_sizer.Add(wx.StaticText(self, label='From:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.units_from_input = wx.TextCtrl(self)
		units_sizer.Add(self.units_from_input)

		units_sizer.Add(wx.StaticText(self, label='To:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.units_to_input = wx.TextCtrl(self)
		units_sizer.Add(self.units_to_input)

		# End buttons.
		button_box = wx.BoxSizer(wx.HORIZONTAL)
		dialog_box.Add(button_box, flag=wx.CENTER)

		ok_button = wx.Button(self, wx.ID_OK)
		self.Bind(wx.EVT_BUTTON, self.OnOk, ok_button)
		button_box.Add(ok_button)

		cancel_button = wx.Button(self, wx.ID_CANCEL)
		button_box.Add(cancel_button)

		self.SetSizerAndFit(dialog_box)

	def OnOk(self, evt=None):
		self.ok_callback(self)

		self.Destroy()

	def GetValue(self):
		plot_settings = PlotSettings()
		plot_settings.enabled = self.enabled_checkbox.Value
		plot_settings.num_points = self.points_input.Value
		plot_settings.delay = Quantity(self.delay_input.GetValue(), 's')
		plot_settings.update_x = self.update_x_axis.Value
		plot_settings.time_value = self.time_value.Selection
		plot_settings.time_mode = self.time_mode.Selection
		plot_settings.update_y = self.update_y_axis.Value
		plot_settings.y_scale = self.y_scale.GetValue()
		plot_settings.units_from = self.units_from_input.Value
		plot_settings.units_to = self.units_to_input.Value

		return plot_settings

	def SetValue(self, plot_settings):
		self.enabled_checkbox.Value = plot_settings.enabled
		self.points_input.Value = plot_settings.num_points
		self.delay_input.SetValue(plot_settings.delay.value)
		self.update_x_axis.Value = plot_settings.update_x
		self.time_value.Selection = plot_settings.time_value
		self.time_mode.Selection = plot_settings.time_mode
		self.update_y_axis.Value = plot_settings.update_y
		self.y_scale.SetValue(plot_settings.y_scale)
		self.units_from_input.Value = plot_settings.units_from
		self.units_to_input.Value = plot_settings.units_to


class ScalarLiveViewPanel(wx.Panel):
	"""
	A panel to display a live view plot of a scalar resource.
	"""

	def __init__(self, parent, global_store, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		self.global_store = global_store
		self._measurement_resource_name = None

		# Defaults.
		self.plot_settings = PlotSettings()
		self.unit_conversion = 0

		self.enabled = False
		self.capturing_data = False
		self.restart_live_view = False
		self.resource_backup = None

		# This lock blocks the acquisition thread from acquiring.
		self.running_lock = Lock()

		# Initialize recorded values.
		self.init_values()

		# Plot and toolbar.
		display_box = wx.BoxSizer(wx.VERTICAL)

		## Plot.
		if plot_available:
			self.plot = TwoDimensionalPlot(self, color='blue')
			display_box.Add(self.plot.control, proportion=1, flag=wx.EXPAND)

			self.plot.x_label = 'Time (s)'
		else:
			display_box.Add((500, -1), proportion=1, flag=wx.EXPAND)

		## Controls.
		if plot_available:
			controls_box = wx.BoxSizer(wx.HORIZONTAL)
			display_box.Add(controls_box, flag=wx.CENTER|wx.ALL, border=5)

			### Numeric display.
			numeric_display_static_box = wx.StaticBox(self, label='Reading')
			numeric_display_box = wx.StaticBoxSizer(numeric_display_static_box, wx.HORIZONTAL)
			controls_box.Add(numeric_display_box, flag=wx.CENTER)

			self.numeric_display = wx.TextCtrl(self, size=(100, -1), style=wx.TE_READONLY)
			self.numeric_display.BackgroundColour = wx.LIGHT_GREY
			numeric_display_box.Add(self.numeric_display)

			### Capture.
			capture_static_box = wx.StaticBox(self, label='Control')
			capture_box = wx.StaticBoxSizer(capture_static_box)
			controls_box.Add(capture_box, flag=wx.CENTER|wx.LEFT, border=10)

			self.run_button = wx.Button(self, label='Run')
			self.Bind(wx.EVT_BUTTON, self.OnRun, self.run_button)
			capture_box.Add(self.run_button, flag=wx.CENTER)

			self.pause_button = wx.Button(self, label='Pause')
			self.Bind(wx.EVT_BUTTON, self.OnPause, self.pause_button)
			capture_box.Add(self.pause_button, flag=wx.CENTER)

			self.reset_button = wx.Button(self, label='Reset')
			self.Bind(wx.EVT_BUTTON, self.OnReset, self.reset_button)
			capture_box.Add(self.reset_button, flag=wx.CENTER|wx.LEFT, border=10)

			### Settings.
			settings_static_box = wx.StaticBox(self, label='Settings')
			settings_box = wx.StaticBoxSizer(settings_static_box, wx.HORIZONTAL)
			controls_box.Add(settings_box, flag=wx.CENTER|wx.LEFT, border=10)

			self.plot_settings_button = wx.Button(self, label='Plot...')
			self.Bind(wx.EVT_BUTTON, self.OnPlotSettings, self.plot_settings_button)
			settings_box.Add(self.plot_settings_button, flag=wx.CENTER)

		self.SetSizer(display_box)

		# Acquisition thread.
		callback = functools.partial(wx.CallAfter, self.add_value)
		self.acq_thread = AcquisitionThread(self.plot_settings.delay, callback,
				running_lock=self.running_lock)
		self.acq_thread.daemon = True
		self.acq_thread.start()

		# Wait for a resource to begin capturing.
		self.OnPause()
		self.run_button.Disable()

		# Subscriptions.
		pub.subscribe(self.msg_resource, 'resource.added')
		pub.subscribe(self.msg_resource, 'resource.removed')
		pub.subscribe(self.msg_data_capture_start, 'data_capture.start')
		pub.subscribe(self.msg_data_capture_data, 'data_capture.data')
		pub.subscribe(self.msg_data_capture_stop, 'data_capture.stop')

	@property
	def running(self):
		return self.pause_button.Enabled

	@property
	def resource(self):
		return self.acq_thread.resource

	@resource.setter
	def resource(self, value):
		# Ignore unreadable resources.
		if value is not None and not value.readable:
			value = None

		if self.running:
			# Currently running.
			running = True
			self.OnPause()
		else:
			running = False

		self.acq_thread.resource = value

		self.run_button.Enable(value is not None)

		# Resume if applicable.
		if running:
			self.OnRun()

	@property
	def measurement_resource_name(self):
		if self._measurement_resource_name is None:
			return ''
		else:
			return self._measurement_resource_name

	@measurement_resource_name.setter
	def measurement_resource_name(self, value):
		if value:
			self._measurement_resource_name = value
			try:
				self.resource = self.global_store.resources[self._measurement_resource_name]
			except KeyError:
				self.resource = None
		else:
			self._measurement_resource_name = None
			self.resource = None

	def init_values(self):
		"""
		Clear captured values.
		"""

		self._points = numpy.array([])
		self._times = numpy.array([])
		self._values = numpy.array([])

		self.current_value = None

		self.start_time = None

	def update_plot(self):
		"""
		Redraw the plot.
		"""

		if not len(self._points) > 0:
			display_time = [0]
			display_values = [0]
		else:
			if self.plot_settings.time_value == 0: # Time.
				display_time = self._times

				if self.plot_settings.time_mode == 0: # Relative.
					# Calculate the number of seconds passed since each point.
					max_time = self._times[-1]
					display_time = [x - max_time for x in display_time]
				elif self.plot_settings.time_mode == 1: # Absolute.
					display_time = [x - self.start_time for x in display_time]
			elif self.plot_settings.time_value == 1: # Points.
				display_time = self._points

				if self.plot_settings.time_mode == 0: # Relative.
					# Calculate the number of seconds passed since each point.
					max_point = self._points[-1]
					display_time = [x - max_point for x in display_time]

			display_values = [x * 10 ** (self.plot_settings.y_scale + self.unit_conversion) for
					x in self._values]

		if self.plot_settings.update_x:
			self.plot.x_autoscale()
		if self.plot_settings.update_y:
			self.plot.y_autoscale()

		self.plot.x_data, self.plot.y_data = display_time, display_values

	def add_value(self, value):
		"""
		Update the plot with a new value.
		"""

		if not self.plot_settings.enabled:
			return

		# Extract the value of a Quantity.
		try:
			value = value.value
		except AttributeError:
			pass

		# Update values.
		try:
			self._points = numpy.append(self._points, self._points[-1] + 1)
		except IndexError:
			self._points = numpy.append(self._points, 0)
		cur_time = time.time()
		self._times = numpy.append(self._times, cur_time)
		self._values = numpy.append(self._values, value)

		if self.start_time is None:
			self.start_time = cur_time

		cut_idx = len(self._points) - int(self.plot_settings.num_points)
		if cut_idx > 0:
			self._points = self._points[cut_idx:]
			self._times = self._times[cut_idx:]
			self._values = self._values[cut_idx:]

		# Set number display.
		self.current_value = value * 10 ** (self.plot_settings.y_scale + self.unit_conversion)
		self.numeric_display.Value = '{0:.6g}'.format(self.current_value)

		# Plot.
		self.update_plot()

	def close(self):
		"""
		Perform cleanup.
		"""

		# Unsubscriptions.
		pub.unsubscribe(self.msg_resource, 'resource.added')
		pub.unsubscribe(self.msg_resource, 'resource.removed')
		pub.unsubscribe(self.msg_data_capture_start, 'data_capture.start')
		pub.unsubscribe(self.msg_data_capture_data, 'data_capture.data')
		pub.unsubscribe(self.msg_data_capture_stop, 'data_capture.stop')

		# Ensure the thread exits.
		self.acq_thread.resource = None
		self.acq_thread.done = True
		if not self.running:
			self.running_lock.release()
		self.acq_thread.join()
		del self.acq_thread

	def OnRun(self, evt=None):
		"""
		Let the acquisition thread run.
		"""

		self.run_button.Disable()

		if self.acq_thread.resource is None:
			return

		self.running_lock.release()

		self.pause_button.Enable()

	def OnPause(self, evt=None):
		"""
		Block the acquisition thread.
		"""

		if not self.running:
			return

		self.running_lock.acquire()

		if self.acq_thread.resource is not None:
			self.run_button.Enable()
		self.pause_button.Disable()

	def OnReset(self, evt=None):
		self.init_values()
		self.update_plot()

	def OnPlotSettings(self, evt=None):
		"""
		Open the plot settings dialog.
		"""

		def ok_callback(dlg):
			self.plot_settings = dlg.GetValue()

			if self.plot_settings.units_from and self.plot_settings.units_to:
				try:
					quantity_from = Quantity(1, self.plot_settings.units_from)
					quantity_to = Quantity(1, self.plot_settings.units_to)
				except ValueError as e:
					self.unit_conversion = 0
					MessageDialog(self, str(e), 'Invalid unit').Show()
				else:
					# We don't actually care about the units; just the prefix values.
					self.unit_conversion = math.log(quantity_from.value, 10) - math.log(quantity_to.value, 10)
			else:
				self.unit_conversion = 0

			self.acq_thread.delay = self.plot_settings.delay

			if self.plot_settings.time_value == 0:
				self.plot.x_label = 'Time (s)'
			elif self.plot_settings.time_value == 1:
				self.plot.x_label = 'Points'

			if self.plot_settings.y_scale != 0:
				self.plot.y_label = '/ 10 ^ {0}'.format(self.plot_settings.y_scale)
			else:
				self.plot.y_label = ''

			if self.plot_settings.units_to:
				self.plot.y_label += ' ({0})'.format(self.plot_settings.units_to)

			self.update_plot()

		dlg = PlotSettingsDialog(self, ok_callback)
		dlg.SetValue(self.plot_settings)
		dlg.Show()

	def msg_resource(self, name, value=None):
		if self.measurement_resource_name is not None and name == self.measurement_resource_name:
			self.resource = value

	def msg_data_capture_start(self, name):
		if name == self.measurement_resource_name:
			if self.enabled:
				self.capturing_data = True

				# Keep track of whether to restart the capture afterwards.
				self.restart_live_view = self.running

				# Disable live view.
				self.resource_backup = self.resource
				self.resource = None

	def msg_data_capture_data(self, name, value):
		if name == self.measurement_resource_name:
			if self.capturing_data:
				self.add_value(value)

	def msg_data_capture_stop(self, name):
		if name == self.measurement_resource_name:
			if self.capturing_data:
				self.capturing_data = False

				# Re-enable live view.
				self.resource = self.resource_backup
				self.resource_backup = None

				if self.restart_live_view:
					self.OnRun()


class ScalarMeasurementFrame(wx.Frame):
	def __init__(self, parent, global_store, *args, **kwargs):
		wx.Frame.__init__(self, parent, *args, **kwargs)

		# Frame.
		frame_box = wx.BoxSizer(wx.VERTICAL)

		## Measurement setup.
		self.measurement_config_panel = MeasurementConfigPanel(self, global_store)
		frame_box.Add(self.measurement_config_panel, flag=wx.EXPAND)

		## Live view.
		self.live_view_panel = ScalarLiveViewPanel(self, global_store)
		self.live_view_panel.SetMinSize((-1, 400))
		frame_box.Add(self.live_view_panel, proportion=1, flag=wx.EXPAND)

		self.SetSizerAndFit(frame_box)

		self.Bind(wx.EVT_CLOSE, self.OnClose)

	def OnClose(self, evt):
		if self.live_view_panel.capturing_data:
			msg = 'Cannot close, as a sweep is currently in progress.'
			MessageDialog(self, msg, 'Sweep in progress').Show()

			evt.Veto()
			return

		self.live_view_panel.close()
		self.measurement_config_panel.close()

		evt.Skip()



from nose.tools import assert_raises, eq_
from StringIO import StringIO
from unittest import main, TestCase

from .. import box


class FindLocationTest(TestCase):
	multiline = [
		'',
		'This is some text.',
		'',
		'It has blank lines, and no tabs.\r',
		'It also has Windows-style line endings.',
		'Sometimes.'
	]

	def testOneLine(self):
		s = 'Test string. Do not read.'

		data = [
			(0, 1, 1),
			(5, 1, 6),
			(25, 1, 26),
		]

		for loc, row, col in data:
			eq_(box.find_location(s, loc), (row, col, s))

	def testMultiline(self):
		s = '\n'.join(self.multiline)
		
		data = [
			(0, 1, 1),
			(40, 4, 20),
			(105, 6, 11),
		]

		for loc, row, col in data:
			eq_(box.find_location(s, loc), (row, col, self.multiline[row - 1].rstrip()))

	def testInvalid(self):
		s = '\n'.join(self.multiline)

		data = [-10, -1, 106]

		for loc in data:
			assert_raises(ValueError, box.find_location, s, loc)


class FormatErrorTest(TestCase):
	def testWithoutPosition(self):
		data = ['', 'test', 'This is a message!']

		for msg in data:
			eq_(box.format_error(msg), 'error: ' + msg)

	def testWithPosition(self):
		result = box.format_error('Test', 1, 2, 'line')

		eq_(result, 'error: Test at column 2 on line 1:\n  line\n   ^')


class LoadValuesTest(TestCase):
	def testEmpty(self):
		f = StringIO()
		result = box.load_values(f)

		eq_(result, [])

	def testOneLine(self):
		f = StringIO('1.0, 2.0, 3.0,4.0')
		result = box.load_values(f)

		eq_(result, [1.0, 2.0, 3.0, 4.0])

	def testOneColumn(self):
		f = StringIO('1.0\n2.0\n3.0\n4.0')
		result = box.load_values(f)

		eq_(result, [1.0, 2.0, 3.0, 4.0])

	def testShaped(self):
		f = StringIO("""
			1.0  ,2.0,3.0

			4.0,	5.0
			6.0

		""")

		result = box.load_values(f)

		eq_(result, [1.0, 2.0, 3.0, 4.0, 5.0, 6.0])


if __name__ == '__main__':
	main()



from nose.tools import eq_
from os import path
from unittest import main, TestCase

from ...units import Quantity
from ..parser import (Acquire, Assignment, Attribute, Block, Declaration,
		Delay, Dictionary, DictionaryItem, Loop, ParallelPulses, Pulse,
		PulseSequence, Variable)

from .. import parser


resource_dir = path.join(path.dirname(__file__), 'resources')


class ParserTest(TestCase):
	def testValid(self):
		"""
		Run down the happy path, line by line.
		"""

		prog = [
			# Lone declaration.
			('int test',
				Declaration({'type': 'int', 'variables': [
					Variable({'name': 'test'})]})),
			# Declaration with some initialization.
			('delay d1=+5ms,d2,\td3 = -7e2   ns2.J',
				Declaration({'type': 'delay', 'variables': [
					Assignment({'target': 'd1', 'value': Quantity(5, 'ms')}),
					Variable({'name': 'd2'}),
					Assignment({'target': 'd3', 'value': Quantity(-7e2, 'ns2.J')})]})),
			# Assignment of value.
			('test = -99',
				Assignment({'target': 'test', 'value': -99})),
			# Assignment of multiple values.
			('long_pulse = {shape: \'filename\', length: 1 Ps}',
				Assignment({'target': 'long_pulse', 'value': Dictionary([
					DictionaryItem({'key': 'shape', 'value': 'filename'}),
					DictionaryItem({'key': 'length', 'value': Quantity(1, 'Ps')})])})),
			# Comments.
			('  output f1 ,f2\t # outputs',
				Declaration({'type': 'output', 'variables': [
					Variable({'name': 'f1'}),
					Variable({'name': 'f2'})]})),
			# Delay.
			('10 us',
				Delay({'length': Quantity(10, 'us')})),
			# Variable delay.
			('d5 #Comment: a = 5',
				Delay({'length': 'd5'})),
			# Attribute assignment.
			('p1.length = 50.0 as',
				Assignment({
					'target': Attribute({'variable': 'p1', 'name': 'length'}),
					'value': Quantity(50.0, 'as')})),
			# Pulse command.
			('some_pulse:f1',
				ParallelPulses([
					Pulse({'sequence': PulseSequence(['some_pulse']), 'target': 'f1'})])),
			# Parallel pulses.
			('a_pulse:outputA another_pulse:outputB',
				ParallelPulses([
					Pulse({'sequence': PulseSequence(['a_pulse']), 'target': 'outputA'}),
					Pulse({'sequence': PulseSequence(['another_pulse']), 'target': 'outputB'})])),
			# Multiple pulse commands.
			('(several commands with a 99 fs delay):formula1 lone_command:indy500',
				ParallelPulses([
					Pulse({'sequence': PulseSequence([
						'several', 'commands', 'with', 'a',
						Delay({'length': Quantity(99, 'fs')}),
						'delay']), 'target': 'formula1'}),
					Pulse({'sequence': PulseSequence(['lone_command']), 'target': 'indy500'})])),
			# Acquire.
			('\tacquire   #',
				Acquire())
		]

		pp = parser.Parser()

		for line, ast in prog:
			try:
				result = pp(line)
			except Exception:
				print line
				raise

			eq_(result, Block([ast]))

	def testMultiple(self):
		"""
		Several lines at once.
		"""

		prog = """
		# Declare:
		int placeholder = 0
		delay settle, end_delay = 10 ns
		pulse first_square, wobble
		output f1, f2

		# Configure:
		first_square = {shape: 'square', length: 1 ms}
		wobble.length = 50 us
		wobble.amplitude = -0.5 mV

		# Execute:
		10 us
		first_square:f1 (wobble +1e-3 us wobble):f2
		settle # Wait before the acquisition.
		acquire
		end_delay
		"""

		expected = [
			Declaration({'type': 'int', 'variables': [
				Assignment({'target': 'placeholder', 'value': 0})]}),
			Declaration({'type': 'delay', 'variables': [
				Variable({'name': 'settle'}),
				Assignment({'target': 'end_delay', 'value': Quantity(10, 'ns')})]}),
			Declaration({'type': 'pulse', 'variables': [
				Variable({'name': 'first_square'}),
				Variable({'name': 'wobble'})]}),
			Declaration({'type': 'output', 'variables': [
				Variable({'name': 'f1'}),
				Variable({'name': 'f2'})]}),
			Assignment({'target': 'first_square',
				'value': Dictionary([
					DictionaryItem({'key': 'shape', 'value': 'square'}),
					DictionaryItem({'key': 'length', 'value': Quantity(1, 'ms')})])}),
			Assignment({
				'target': Attribute({'variable': 'wobble', 'name': 'length'}),
				'value': Quantity(50, 'us')}),
			Assignment({
				'target': Attribute({'variable': 'wobble', 'name': 'amplitude'}),
				'value': Quantity(-0.5, 'mV')}),
			Delay({'length': Quantity(10, 'us')}),
			ParallelPulses([
				Pulse({'sequence': PulseSequence(['first_square']), 'target': 'f1'}),
				Pulse({'sequence': PulseSequence([
					'wobble',
					Delay({'length': Quantity(+1e-3, 'us')}),
					'wobble']), 'target': 'f2'})]),
				Delay({'length': 'settle'}),
			Acquire(),
			Delay({'length': 'end_delay'}),
		]

		pp = parser.Parser()
		result = pp(prog)

		eq_(result, Block(expected))

	def testLoop(self):
		"""
		Repeat!
		"""

		prog = """
		times 5 {
			d1
			p1:f1 p2:f2
		}

		times M {
			d2

			times N {
				p2:f1 p1:f2
			}
		}
		"""

		expected = [
			Loop({'times': 5, 'block': Block([
				Delay({'length': 'd1'}),
				ParallelPulses([
					Pulse({'sequence': PulseSequence(['p1']), 'target': 'f1'}),
					Pulse({'sequence': PulseSequence(['p2']), 'target': 'f2'})])])}),
			Loop({'times': 'M', 'block': Block([
				Delay({'length': 'd2'}),
				Loop({'times': 'N', 'block': Block([
					ParallelPulses([
						Pulse({'sequence': PulseSequence(['p2']), 'target': 'f1'}),
						Pulse({'sequence': PulseSequence(['p1']), 'target': 'f2'})])])})])}),
		]

		pp = parser.Parser()
		result = pp(prog)

		eq_(result, Block(expected))

	def testOneLine(self):
		"""
		Line breaks are optional.
		"""

		prog = 'int x=5;delay d1;;d1;d1;times x{d1;d1;};times x{d1}d1'

		expected = [
			Declaration({'type': 'int', 'variables': [
				Assignment({'target': 'x', 'value': 5})]}),
			Declaration({'type': 'delay', 'variables': [
				Variable({'name': 'd1'})]}),
			Delay({'length': 'd1'}),
			Delay({'length': 'd1'}),
			Loop({'times': 'x', 'block': Block([
				Delay({'length': 'd1'}),
				Delay({'length': 'd1'})])}),
			Loop({'times': 'x', 'block': Block([
				Delay({'length': 'd1'})])}),
			Delay({'length': 'd1'}),
		]

		pp = parser.Parser()
		result = pp(prog)

		eq_(result, Block(expected))

	def testFromFile(self):
		"""
		Parse an entire file.
		"""

		expected = [
			Declaration({'type': 'int', 'variables': [
				Assignment({'target': 'bumps', 'value': 2})]}),
			Declaration({'type': 'delay', 'variables': [
				Assignment({'target': 'bump_spacing', 'value': Quantity(10, 'ns')}),
				Variable({'name': 'settle'}),
				Assignment({'target': 'end_delay', 'value': Quantity(15, 'ns')})]}),
			Declaration({'type': 'pulse', 'variables': [
				Variable({'name': 'first_square'}),
				Variable({'name': 'wobble'})]}),
			Declaration({'type': 'pulse', 'variables': [
				Assignment({'target': 'last_square', 'value': Dictionary([
					DictionaryItem({'key': 'shape', 'value': 'square'})])})]}),
			Declaration({'type': 'pulse', 'variables': [
				Assignment({'target': 'manipulator', 'value': Dictionary([
					DictionaryItem({'key': 'shape', 'value': 'non-square'})])})]}),
			Declaration({'type': 'output', 'variables': [
				Variable({'name': 'f1'}),
				Variable({'name': 'f2'})]}),
			Assignment({'target': 'first_square', 'value': Dictionary([
				DictionaryItem({'key': 'shape', 'value': 'square'}),
				DictionaryItem({'key': 'length', 'value': Quantity(1, 'ns')})])}),
			Assignment({
				'target': Attribute({'variable': 'wobble', 'name': 'length'}),
				'value': Quantity(8, 'ns')}),
			Assignment({
				'target': Attribute({'variable': 'wobble', 'name': 'amplitude'}),
				'value': Quantity(-1, 'mV')}),
			Delay({'length': Quantity(10, 'ns')}),
			ParallelPulses([
				Pulse({'sequence': PulseSequence(['first_square']), 'target': 'f1'}),
				Pulse({'sequence': PulseSequence(['wobble']), 'target': 'f2'})]),
			Loop({'times': 'bumps', 'block': Block([
				Delay({'length': 'bump_spacing'}),
				Loop({'times': 2, 'block': Block([
					ParallelPulses([
						Pulse({'sequence': PulseSequence(['first_square']), 'target': 'f1'}),
						Pulse({'sequence': PulseSequence(['wobble']), 'target': 'f2'})])])})])}),
					Delay({'length': 'settle'}),
			Acquire(),
			ParallelPulses([
				Pulse({'sequence': PulseSequence(['last_square']), 'target': 'f1'}),
				Pulse({'sequence': PulseSequence(['manipulator', 'end_delay', 'manipulator']), 'target': 'f2'})]),
			Delay({'length': 'end_delay'}),
		]

		pp = parser.Parser()

		with open(path.join(resource_dir, '01.pulse')) as f:
			result = pp(''.join(f.readlines()))

		eq_(result, Block(expected))

	def testInvalid(self):
		"""
		Try the error detection.
		"""

		prog = [
			'int a b', 'int a = b = c', '5:f1', 'times {d1;d2}', 'd1;{d2}d3',
			'a = b', 'c.d = e.f', 'g.h', 'd1 (d2 d3):f1', 'refresh = d1:f5',
			'times 5 V {}', '100 units',
		]

		pp = parser.Parser()

		for line in prog:
			try:
				pp(line)
			except parser.PulseSyntaxError:
				pass
			else:
				assert False, 'Expected ParseException'


if __name__ == '__main__':
	main()



import logging
log = logging.getLogger(__name__)

from copy import copy
from numpy import linspace
from threading import Thread
import time

from .units import IncompatibleDimensions, Quantity

from spacq.tool.box import Without

"""
Tools for working with generic resources.
"""


class NotReadable(Exception):
	"""
	Resource cannot be read from.
	"""

	pass


class NotWritable(Exception):
	"""
	Resource cannot be written to.
	"""

	pass


class Resource(object):
	"""
	A generic resource which can potentially be read from or written to.
	"""

	def __init__(self, obj=None, getter=None, setter=None, converter=None, allowed_values=None):
		"""
		obj: The device to which the resource belongs.
		getter: The method used to get the value.
		setter: The method used to set the value.
		converter: A function which returns a valid value for the resource, given a string.
		allowed_values: A set of all values which are valid for the resource.

		The getter and setter can be actual methods, or just attribute/property name strings.
		"""

		if getter is not None and not callable(getter) and obj is None:
			raise ValueError('Cannot call getter with no object.')

		if setter is not None and not callable(setter) and obj is None:
			raise ValueError('Cannot call setter with no object.')

		self.obj = obj
		self.getter = getter
		self.setter = setter
		self.converter = converter
		if allowed_values is not None:
			self.allowed_values = set(allowed_values)
		else:
			self.allowed_values = None

		self.wrappers = []

		# Dimensions (specified as unit symbol strings) which values for this resource must match.
		self._units = None
		# String used for display purposes. Typically the same as self.units.
		self.display_units = None

		# Resources marked slow should not be fetched implicitly.
		self.slow = False

	@property
	def units(self):
		return self._units

	@units.setter
	def units(self, value):
		self._units = value

		if self.display_units is None:
			self.display_units = self.units

	def verify_dimensions(self, value, exception=True, from_string=False):
		"""
		Ensure that the type and dimensions of the value are as expected.

		If from_string is True, the value is expected to be a unit symbol string.
		"""

		if from_string:
			value = Quantity(1, value)

		try:
			if self.units is not None:
				try:
					value.assert_dimensions(self.units)
				except AttributeError:
					raise TypeError('Expected a Quantity, not "{0}"'.format(value))
				except IncompatibleDimensions:
					raise TypeError('Expected dimensions matching "{0}", not "{1}"'.format(self.units, value))
			else:
				if isinstance(value, Quantity):
					raise TypeError('Unexpected Quantity "{0}"'.format(value))
		except Exception:
			if exception:
				raise
			else:
				return False

		return True

	@property
	def value(self):
		"""
		The value of the resource.
		"""

		if self.getter is None:
			raise NotReadable('Resource not readable.')

		if callable(self.getter):
			result = self.getter()
		elif self.obj is not None:
			result = getattr(self.obj, self.getter)
		else:
			raise NotReadable('Cannot read from resource.')

		self.verify_dimensions(result)

		# Apply the wrappers.
		for _, getter_filter, _ in self.wrappers:
			if getter_filter is None:
				continue

			result = getter_filter(result)
			self.verify_dimensions(result)

		return result

	@value.setter
	def value(self, v):
		if self.setter is None:
			raise NotWritable('Resource not writable.')

		self.verify_dimensions(v)

		# Apply the wrappers.
		for _, _, setter_filter in self.wrappers:
			if setter_filter is None:
				continue

			v = setter_filter(v)
			self.verify_dimensions(v)

		if self.allowed_values is not None and v not in self.allowed_values:
			raise ValueError('Disallowed value: {0}'.format(v))

		if callable(self.setter):
			self.setter(v)
		elif self.obj is not None:
			setattr(self.obj, self.setter, v)
		else:
			raise NotWritable('Cannot write to resource.')

	def convert(self, value):
		"""
		Either use the specified converter, treat as a quantity, or do nothing.
		"""

		if self.converter is not None:
			return self.converter(value)
		elif self.units is not None:
			q = Quantity(value)
			q.assert_dimensions(self.units)

			return q
		else:
			return value

	@property
	def readable(self):
		return self.getter is not None

	@property
	def writable(self):
		return self.setter is not None

	def _find_wrapper_by_name(self, name):
		"""
		Return the last index of the given wrapper.
		"""

		for i, (n, _, _) in reversed(list(enumerate(self.wrappers))):
			if n == name:
				return i

		raise ValueError('Wrapper not found: {0}'.format(name))

	def is_wrapped_by(self, name):
		"""
		Determine whether the given wrapper already wraps this Resource.
		"""

		try:
			self._find_wrapper_by_name(name)
		except ValueError:
			return False
		else:
			return True

	def wrapped(self, name, getter_filter=None, setter_filter=None):
		"""
		Produce a Resource which is a wrapper around this Resource.

		name: The name of the wrapper to add.
		getter_filter: Function of one argument through which to pass any obtained values.
		setter_filter: Function of one argument through which to pass values when setting.
		"""

		result = copy(self)

		result.wrappers = self.wrappers + [(name, getter_filter, setter_filter)]

		return result

	def unwrapped(self, name):
		"""
		Produce a Resource with the last instance of the given wrapper removed.

		name: The name of the wrapper to remove.
		"""

		result = copy(self)

		idx = self._find_wrapper_by_name(name)
		result.wrappers = self.wrappers[:idx] + self.wrappers[idx+1:]

		return result

	def sweep(self, value_from, value_to, steps, delay=0.1, exception_callback=None):
		"""
		Sweep the Resource slowly over a linear space.
		"""

		# Check for dimension mismatches.
		if isinstance(value_from, Quantity) and not isinstance(value_to, Quantity) and value_to == 0:
			value_to = Quantity(0, value_from.original_units)
		elif isinstance(value_to, Quantity) and not isinstance(value_from, Quantity) and value_from == 0:
			value_from = Quantity(0, value_to.original_units)
		elif isinstance(value_from, Quantity) and isinstance(value_to, Quantity):
			value_from.assert_dimensions(value_to)

		for value in linspace(value_from, value_to, steps):
			try:
				self.value = value
			except Exception as e:
				if exception_callback is not None:
					exception_callback(e)
				return

			time.sleep(delay)


class AcquisitionThread(Thread):
	"""
	Once every delay, call the callback with a fresh value from the resource.

	An optional running lock can block execution until it is released elsewhere.
	"""

	def __init__(self, delay, callback, resource=None, running_lock=None):
		Thread.__init__(self)

		delay.assert_dimensions('s')

		self.resource = resource
		self.delay = delay
		self.callback = callback
		if running_lock is None:
			self.running_lock = Without()
		else:
			self.running_lock = running_lock

		# Allow the thread to be stopped prematurely.
		self.done = False

	def run(self):
		while not self.done:
			# If something goes wrong, sleep the maximum.
			delay = self.delay.value

			with self.running_lock:
				if time:
					next_run = time.time() + self.delay.value
				else:
					# Weird things happen at shutdown.
					return

				if self.resource is not None:
					try:
						value = self.resource.value
					except Exception as e:
						log.error('Could not obtain resource value: {0!r}'.format(e))
					else:
						self.callback(value)

				if time:
					delay = next_run - time.time()
				else:
					return

			if not self.done and delay > 0:
				time.sleep(delay)



import struct

from ...mock.mock_abstract_device import MockAbstractDevice
from ...tools import BlockData
from ..awg5014b import AWG5014B

"""
Mock Tektronix AWG5014B Arbitrary Waveform Generator

Control a fake AWG's settings and output waveforms.
"""


class Waveform(object):
	"""
	Mock waveform.
	"""

	def __init__(self, name, length):
		self.name = name
		self.length = length
		self._data = [0] * length

	@property
	def data(self):
		return self._data

	@data.setter
	def data(self, v):
		new_length = len(v)

		if new_length <= self.length:
			self._data[:new_length] = v


class MockMarker(object):
	"""
	A mock marker for a mock channel.
	"""

	def __init__(self):
		self.delay = 0.0 # s
		self.low = 0.0 # V
		self.high = 1.0 # V


class MockChannel(object):
	"""
	A mock channel for a mock AWG.
	"""

	def __init__(self):
		self.enabled = False
		self.waveform_name = ''
		self.voltage = 1.0

		self.markers = [None] # There is no marker 0.
		for _ in xrange(1, 3):
			self.markers.append(MockMarker())


class MockAWG5014B(MockAbstractDevice, AWG5014B):
	"""
	Mock interface for Tektronix AWG5014B AWG.
	"""

	def __init__(self, *args, **kwargs):
		"""
		Pretend to connect to the AWG, but do initialize with some values.
		"""

		self.mocking = AWG5014B

		MockAbstractDevice.__init__(self, *args, **kwargs)

	def _reset(self):
		self.mock_state['run_mode'] = 'continuous'
		self.mock_state['run_state'] = '0'
		self.mock_state['frequency'] = 1.2e9 # Hz

		self.mock_state['wlist'] = []
		self.mock_state['wlist'].append(Waveform('"predefined waveform"', 5))
		self.mock_state['wlist'][0].data = list(xrange(5))

		self.mock_state['channels'] = [None] # There is no channel 0.
		for _ in xrange(1, 5):
			self.mock_state['channels'].append(MockChannel())

	def find_wave(self, name):
		"""
		Find a Waveform object by name.
		"""

		for wave in self.mock_state['wlist']:
			if wave.name == name:
				return wave

	def write(self, message, result=None, done=False):
		if not done:
			cmd, args, query = self._split_message(message)

			if cmd[0] == '*trg':
				done = True
			elif cmd[0] == 'awgcontrol':
				if cmd[1] == 'run':
					if self.mock_state['run_mode'] in ['triggered', 'gated']:
						self.mock_state['run_state'] = '1'
					else:
						self.mock_state['run_state'] = '2'
					done = True
				elif cmd[1] == 'stop':
					self.mock_state['run_state'] = '0'
					done = True
				elif cmd[1] == 'rstate' and query:
					result = self.mock_state['run_state']
					done = True
				elif cmd[1] == 'rmode':
					if query:
						result = self.mock_state['run_mode']
						done = True
					else:
						mode = args
						self.mock_state['run_mode'] = mode

						if self.mock_state['run_state'] != '0':
							if mode in ['triggered', 'gated']:
								self.mock_state['run_state'] = '1'
							else:
								self.mock_state['run_state'] = '2'
						done = True
			elif cmd[0] == 'wlist':
				if cmd[1] == 'size' and query:
					result = str(len(self.mock_state['wlist']))
					done = True
				elif cmd[1] == 'name' and query:
					num = args
					name = self.mock_state['wlist'][int(num)].name
					result = '"{0}"'.format(name[1:-1])
					done = True
				elif cmd[1] == 'waveform' and cmd[2] == 'new':
					name, length, kind = [arg.strip() for arg in args.split(',')]
					length = int(length)

					if kind == 'integer':
						self.mock_state['wlist'].append(Waveform(name, length))
						done = True
				elif cmd[1] == 'waveform' and cmd[2] == 'data':
					if query:
						name = args

						wave = self.find_wave(name)

						data = wave.data
						data = [datum + marker * 2 ** 14 for (datum, marker) in zip(data, wave.marker1)]
						data = [datum + marker * 2 ** 15 for (datum, marker) in zip(data, wave.marker2)]

						waveform_length = len(data)
						packed_data = struct.pack('<{0}H'.format(waveform_length), *data)
						result = BlockData.to_block_data(packed_data)
					else:
						name, block_data = args.split(',', 1)
						name = name.strip()
						block_data = block_data.lstrip()
						packed_data = BlockData.from_block_data(block_data)
						waveform_length = len(packed_data) / 2
						data = struct.unpack('<{0}H'.format(waveform_length), packed_data)

						wave = self.find_wave(name)
						wave.data = [x & 2 ** 14 - 1 for x in data]
						wave.marker1 = [1 if x & 2 ** 14 else 0 for x in data]
						wave.marker2 = [1 if x & 2 ** 15 else 0 for x in data]

					done = True
				elif cmd[1] == 'waveform' and cmd[2] == 'delete':
					self.mock_state['wlist'].remove(self.find_wave(args))
					done = True
			elif cmd[0].startswith('source'):
				source = int(cmd[0][6])
				channel = self.mock_state['channels'][source]

				if cmd[1] == 'waveform':
					if query:
						result = channel.waveform_name
					else:
						name = args
						channel.waveform_name = name
						if name == '""':
							channel.enabled = False
					done = True
				elif cmd[1] == 'frequency':
					if query:
						result = self.mock_state['frequency']
					else:
						frequency = args
						self.mock_state['frequency'] = float(frequency)
					done = True
				elif cmd[1] == 'voltage':
					if query:
						result = channel.voltage
					else:
						voltage = args
						channel.voltage = float(voltage)
					done = True
				elif cmd[1].startswith('marker'):
					marker_num = int(cmd[1][6])
					marker = channel.markers[marker_num]

					if cmd[2] == 'delay':
						if query:
							result = marker.delay
						else:
							delay = args
							marker.delay = float(delay)
						done = True
					elif cmd[2] == 'voltage' and cmd[3] == 'high':
						if query:
							result = marker.high
						else:
							voltage = args
							marker.high = float(voltage)
						done = True
					elif cmd[2] == 'voltage' and cmd[3] == 'low':
						if query:
							result = marker.low
						else:
							voltage = args
							marker.low = float(voltage)
						done = True
			elif cmd[0].startswith('output'):
				output = int(cmd[0][6])
				channel = self.mock_state['channels'][output]

				if cmd[1] == 'state':
					if query:
						result = str(int(channel.enabled))
					else:
						state = args
						channel.enabled = (state == '1')
					done = True

		MockAbstractDevice.write(self, message, result, done)


name = 'AWG5014B'
implementation = MockAWG5014B



from nose.tools import eq_
from unittest import main, TestCase

from spacq.interface.resources import Resource

from .. import abstract_device


class AbstractDeviceTest(TestCase):
	def testInitNoAddress(self):
		"""
		No address specified.
		"""

		try:
			abstract_device.AbstractDevice()
		except ValueError:
			pass
		else:
			assert False, 'Expected ValueError.'

	def testInitNotFoundIP(self):
		"""
		Invalid or non-existent IP address.
		"""

		try:
			abstract_device.AbstractDevice(ip_address='1234')
		except abstract_device.DeviceNotFoundError:
			pass
		else:
			assert False, 'Expected DeviceNotFoundError.'

		try:
			# Address within TEST-NET-1 is not likely to exist.
			abstract_device.AbstractDevice(ip_address='192.0.2.123')
		except abstract_device.DeviceNotFoundError:
			pass
		else:
			assert False, 'Expected DeviceNotFoundError.'


	def testInitNotFoundGPIB(self):
		"""
		Invalid or non-existent GPIB address.

		Note: There doesn't seem to be a way of disabling the error output from libgpib.
		"""

		try:
			# Valid PADs are on [0, 30] (5 bits; 31 is reserved).
			abstract_device.AbstractDevice(gpib_board=0, gpib_pad=2000)
		except abstract_device.DeviceNotFoundError:
			pass
		else:
			assert False, 'Expected DeviceNotFoundError.'

		try:
			# Assuming that board number 15 is not used.
			abstract_device.AbstractDevice(gpib_board=15, gpib_pad=0)
		except abstract_device.DeviceNotFoundError:
			pass
		else:
			assert False, 'Expected DeviceNotFoundError.'

	def testInitNotFoundUSB(self):
		"""
		Invalid or non-existent USB address.
		"""

		try:
			abstract_device.AbstractDevice(usb_resource='NOT::USB::RAW')
		except abstract_device.DeviceNotFoundError:
			pass
		else:
			assert False, 'Expected DeviceNotFoundError.'

		try:
			# Unlikely VID/PID/serial combination.
			abstract_device.AbstractDevice(usb_resource='USB::1234::5678::01234567::RAW')
		except abstract_device.DeviceNotFoundError:
			pass
		else:
			assert False, 'Expected DeviceNotFoundError.'

	def testFindResource(self):
		"""
		Attempt to find a resource.
		"""

		dev = abstract_device.AbstractDevice(ip_address='192.0.2.123', autoconnect=False)
		subdev1 = abstract_device.AbstractSubdevice(dev)
		subdev2 = abstract_device.AbstractSubdevice(dev)
		subdev3 = abstract_device.AbstractSubdevice(subdev1)
		res1 = Resource(object(), '__class__')
		res2 = Resource(object(), '__str__')
		res3 = Resource(object(), '__doc__')

		dev.subdevices['subdev1'] = subdev1
		dev.resources['res1'] = res1
		dev.subdevices['subdev2'] = subdev2
		subdev1.subdevices['subdev3'] = subdev3
		subdev3.resources['res2'] = res2
		subdev3.resources['res3'] = res3

		# Success.
		found = dev.find_resource(('subdev1', 'subdev3', 'res3'))
		eq_(found.value, object.__doc__)

		# No such resource.
		try:
			dev.find_resource(('subdev1', 'res3'))
		except ValueError:
			pass
		else:
			assert False, 'Expected ValueError.'

		# No such subdevice.
		try:
			dev.find_resource(('subdev1', 'subdev2', 'res3'))
		except ValueError:
			pass
		else:
			assert False, 'Expected ValueError.'

		# Nothing to even try.
		try:
			dev.find_resource(())
		except ValueError:
			pass
		else:
			assert False, 'Expected ValueError.'


if __name__ == '__main__':
	main()



from nose.tools import assert_raises, eq_
from unittest import main, TestCase

from .. import list_columns


class ListParserTest(TestCase):
	def testSmall(self):
		"""
		Try a small list.
		"""

		lp = list_columns.ListParser()

		eq_(lp('[(1.2, 3.4)]'), [(1.2, 3.4)])

	def testLarger(self):
		"""
		Try a larger (Python-generated) list.
		"""

		lst = [(float(x) / 3, float(y) / 5) for x in xrange(55) for y in xrange(25)]

		lp = list_columns.ListParser()

		eq_(lp(str(lst)), lst)

	def testInvalid(self):
		"""
		These aren't lists!
		"""

		lsts = [
			'',
			'[]',
			'(1,2),(3,4)',
			'[(1,2),',
			'[(1,2,3),(4,5,6)]',
		]

		lp = list_columns.ListParser()

		for lst in lsts:
			assert_raises(ValueError, lp, lst)


if __name__ == '__main__':
	main()



import logging
log = logging.getLogger(__name__)

from collections import namedtuple
from time import sleep

from spacq.interface.resources import Resource
from spacq.tool.box import Synchronized

from ..abstract_device import AbstractDevice
from ..tools import str_to_bool, quantity_wrapped, quantity_unwrapped

"""
Oxford Instruments IPS120-10 Superconducting Magnet Power Supply
"""


Status = namedtuple('Status', 'system_status, limits, activity, remote_status, heater, mode, mode_sweep')


class IPS120_10(AbstractDevice):
	"""
	Interface for the Oxford Instruments IPS120-10.
	"""

	allowed_settings = ['default value', 'something else']

	activities = ['hold', 'to_set', 'to_zero', 'clamped']

	heater_delay = 10 # s

	def _setup(self):
		AbstractDevice._setup(self)

		self._perma_hot = True

		# Resources.
		read_write = ['perma_hot', 'sweep_rate', 'field']
		for name in read_write:
			self.resources[name] = Resource(self, name, name)

		self.resources['perma_hot'].converter = str_to_bool
		self.resources['sweep_rate'].units = 'T.s-1'
		self.resources['field'].units = 'T'

	@Synchronized()
	def _connected(self):
		self.eos_char = '\r'

		AbstractDevice._connected(self)

		self.write('$Q4') # Extended resolution.
		self.write('$C3') # Remote & unlocked.
		self.write('$M9') # Display in Tesla.

		if self.device_status.activity == 4:
			self.write('$A0') # Unclamp.

		# Ensure some initial sanity.
		assert self.device_status.activity == 0, 'Not on hold.'

	def write(self, message):
		"""
		Append the "\r" that the device requires.
		"""

		AbstractDevice.write(self, message + '\r')

	@property
	def device_status(self):
		"""
		All the status information for the device.
		"""

		result = self.ask('X')

		system_status = int(result[1])
		limits = int(result[2])
		activity = int(result[4])
		remote_status = int(result[6])
		heater = int(result[8])
		mode = int(result[10])
		mode_sweep = int(result[11])
		# The polarity status is deprecated.

		return Status(system_status, limits, activity, remote_status, heater, mode, mode_sweep)

	@property
	def activity(self):
		"""
		What the device is currently up to.
		"""

		return self.activities[self.device_status.activity]

	@activity.setter
	def activity(self, value):
		self.write('$A{0}'.format(self.activities.index(value)))

	@property
	def heater_on(self):
		"""
		Whether the heater is enabled.
		"""

		return bool(self.device_status.heater & 1)

	@heater_on.setter
	def heater_on(self, value):
		self.status.append('Turning heater o{0}'.format('n' if value else 'ff'))

		try:
			self.write('$H{0}'.format(int(value)))

			# Allow the heater to go to the correct setting.
			log.debug('Waiting for heater for {0} s.'.format(self.heater_delay))
			sleep(self.heater_delay)
		finally:
			self.status.pop()

	@property
	def perma_hot(self):
		"""
		Whether the heater should always remain on.
		"""

		return self._perma_hot

	@perma_hot.setter
	def perma_hot(self, value):
		self._perma_hot = value

	@property
	# The value used on the device is in T/min.
	@quantity_wrapped('T.s-1', 1./60)
	def sweep_rate(self):
		"""
		The rate of the field sweep, as a quantity in T/s.
		"""

		return float(self.ask('R9')[1:])

	@sweep_rate.setter
	@quantity_unwrapped('T.s-1', 60)
	def sweep_rate(self, value):
		if value <= 0:
			raise ValueError('Sweep rate must be positive, not {0}.'.format(value))

		self.write('$T{0:f}'.format(value))

	@property
	@quantity_wrapped('T')
	def persistent_field(self):
		"""
		The output field when the heater was last disabled, as a quantity in T.
		"""

		return float(self.ask('R18')[1:])

	@property
	@quantity_wrapped('T')
	def output_field(self):
		"""
		The actual field due to the output current in T.
		"""

		return float(self.ask('R7')[1:])

	@property
	@quantity_wrapped('T')
	def set_point(self):
		"""
		The set point, as a quantity in T.
		"""

		return float(self.ask('R8')[1:])

	@set_point.setter
	@quantity_unwrapped('T')
	def set_point(self, value):
		self.write('$J{0}'.format(value))

	@property
	def field(self):
		"""
		The magnetic field, as a quantity in T.
		"""

		return self.output_field

	def set_field(self, value):
		"""
		Go through all the steps for setting the output field.
		"""

		if self.output_field == value:
			return

		self.status.append('Setting field to {0}'.format(value))

		try:
			set_delay = abs(value - self.output_field).value / self.sweep_rate.value # s

			self.set_point = value
			self.activity = 'to_set'

			# If the heater is on, the sweep rate is used, so wait.
			if self.heater_on:
				log.debug('Waiting for sweep for {0} s.'.format(set_delay))
				sleep(set_delay)

			# Ensure that the sweep is actually over.
			while self.device_status.mode_sweep != 0:
				sleep(0.1)

			self.activity = 'hold'
		finally:
			self.status.pop()

	@field.setter
	@Synchronized()
	def field(self, value):
		status = self.device_status
		assert status.system_status == 0, 'System status: {0}'.format(status.system_status)
		assert status.limits == 0, 'Limits: {0}'.format(status.limits)
		assert status.mode_sweep == 0, 'Mode sweep: {0}'.format(status.mode_sweep)
		assert self.activity == 'hold', 'Activity: {0}'.format(self.activity)

		# Return to the last field.
		if not self.heater_on:
			self.set_field(self.persistent_field)
			self.heater_on = True

		# Change to the new field.
		self.set_field(value)

		if not self.perma_hot:
			self.heater_on = False

	@property
	def idn(self):
		"""
		*idn? substitute for this non-SCPI device.
		"""

		return self.ask('V')

	@property
	def opc(self):
		"""
		*opc? substitute for this non-SCPI device.
		"""

		return 1


name = 'IPS120-10'
implementation = IPS120_10



from pubsub import pub
import wx

from spacq.iteration.variables import InputVariable

from ..tool.box import OK_BACKGROUND_COLOR, MessageDialog
from .scaling import ScalingSettings, ScalingSettingsDialog


class MeasurementConfigPanel(wx.Panel):
	"""
	Measurement configuration panel.
	"""

	def __init__(self, parent, global_store, scaling=True, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		self.parent = parent
		self.global_store = global_store
		self.scaling = scaling

		if self.scaling:
			self.scaling_settings = ScalingSettings()

		# Ensure that we get a unique name.
		with self.global_store.variables.lock:
			num = 1
			done = False
			while not done:
				name = 'New measurement {0}'.format(num)
				self.var = InputVariable(name=name)

				try:
					self.global_store.variables[name] = self.var
				except KeyError:
					num += 1
				else:
					done = True

		# Keep track of the scaling wrapper and resource.
		if self.scaling:
			self.scaling_wrap_token = '{0}.{1}'.format(self.__class__.__name__, self.wrap_with_scaling.__name__)
		self.resource = None
		self.unwrapping = False

		# Panel.
		panel_box = wx.BoxSizer(wx.VERTICAL)

		## Configuration.
		configuration_box = wx.BoxSizer(wx.HORIZONTAL)
		panel_box.Add(configuration_box, flag=wx.EXPAND|wx.ALL, border=5)

		self.enabled_checkbox = wx.CheckBox(self, label='Capture')
		self.enabled_checkbox.Value = self.var.enabled
		configuration_box.Add(self.enabled_checkbox, flag=wx.CENTER|wx.RIGHT, border=15)

		self.Bind(wx.EVT_CHECKBOX, self.OnCaptureChecked, self.enabled_checkbox)

		### Names.
		names_box = wx.FlexGridSizer(rows=2, cols=2, hgap=5)
		names_box.AddGrowableCol(1, 1)
		configuration_box.Add(names_box, flag=wx.EXPAND, proportion=1)

		names_box.Add(wx.StaticText(self, label='Resource name:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.resource_name_input = wx.TextCtrl(self, value=self.var.resource_name, style=wx.TE_PROCESS_ENTER)
		self.resource_name_input.default_background_color = self.resource_name_input.BackgroundColour
		self.resource_name_input.BackgroundColour = OK_BACKGROUND_COLOR
		names_box.Add(self.resource_name_input, flag=wx.EXPAND)

		self.Bind(wx.EVT_TEXT, self.OnResourceNameChange, self.resource_name_input)
		self.Bind(wx.EVT_TEXT_ENTER, self.OnResourceNameInput, self.resource_name_input)

		names_box.Add(wx.StaticText(self, label='Measurement name:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.measurement_name_input = wx.TextCtrl(self, value=self.var.name, style=wx.TE_PROCESS_ENTER)
		self.measurement_name_input.default_background_color = self.measurement_name_input.BackgroundColour
		self.measurement_name_input.BackgroundColour = OK_BACKGROUND_COLOR
		names_box.Add(self.measurement_name_input, flag=wx.EXPAND)

		self.Bind(wx.EVT_TEXT, self.OnMeasurementNameChange, self.measurement_name_input)
		self.Bind(wx.EVT_TEXT_ENTER, self.OnMeasurementNameInput, self.measurement_name_input)

		### Scaling.
		if self.scaling:
			scaling_button = wx.Button(self, label='Scaling...', style=wx.BU_EXACTFIT)
			self.Bind(wx.EVT_BUTTON, self.OnScaling, scaling_button)
			configuration_box.Add(scaling_button, flag=wx.EXPAND|wx.LEFT, border=10)

		self.SetSizerAndFit(panel_box)

		self.set_title()

		# Subscriptions.
		if self.scaling:
			pub.subscribe(self.msg_resource, 'resource.added')
			pub.subscribe(self.msg_resource, 'resource.removed')

	@property
	def live_view_panel(self):
		return self.parent.live_view_panel

	def wrap_with_scaling(self, name, resource):
		if not self.scaling:
			return

		# Don't double-wrap.
		if resource.is_wrapped_by(self.scaling_wrap_token):
			return

		# Modify the resource value by the scaling.
		def transform(x):
			# Close over self, so that updating scaling settings automatically takes effect.
			return self.scaling_settings.transform(x)
		wrapped_resource = resource.wrapped(self.scaling_wrap_token, transform)

		with self.global_store.lock:
			del self.global_store.resources[name]
			self.global_store.resources[name] = wrapped_resource

	def unwrap_with_scaling(self):
		if not self.scaling:
			return

		if self.resource is None:
			return

		# Don't allow immediate re-wrapping.
		self.unwrapping = True

		name = self.live_view_panel.measurement_resource_name
		unwrapped_resource = self.resource.unwrapped(self.scaling_wrap_token)

		with self.global_store.lock:
			del self.global_store.resources[name]
			self.global_store.resources[name] = unwrapped_resource

		self.resource = None
		self.unwrapping = False

	def set_title(self):
		self.parent.Title = '{0} ({1}){2}'.format(self.var.name, self.var.resource_name,
				'' if self.var.enabled else ' [Disabled]')

	def close(self):
		self.unwrap_with_scaling()

		del self.global_store.variables[self.var.name]

	def OnCaptureChecked(self, evt=None):
		self.var.enabled = self.live_view_panel.enabled = self.enabled_checkbox.Value
		self.set_title()

	def OnResourceNameChange(self, evt=None):
		self.resource_name_input.BackgroundColour = self.resource_name_input.default_background_color

	def OnResourceNameInput(self, evt=None):
		if self.var.resource_name != self.resource_name_input.Value:
			# Ensure that the resource is unwrapped before releasing it.
			self.unwrap_with_scaling()

			self.var.resource_name = name = self.resource_name_input.Value

			# Inform the panel.
			self.live_view_panel.measurement_resource_name = name

			# Grab the new resource if it already exists.
			try:
				self.resource = self.global_store.resources[name]
			except KeyError:
				pass
			else:
				self.wrap_with_scaling(name, self.resource)

		self.resource_name_input.BackgroundColour = OK_BACKGROUND_COLOR
		self.set_title()

	def OnMeasurementNameChange(self, evt=None):
		self.measurement_name_input.BackgroundColour = self.measurement_name_input.default_background_color

	def OnMeasurementNameInput(self, evt=None):
		if self.var.name != self.measurement_name_input.Value:
			# Attempt to add a new entry first.
			var_new_name = self.measurement_name_input.Value
			try:
				self.global_store.variables[var_new_name] = self.var
			except KeyError:
				MessageDialog(self, var_new_name, 'Variable name conflicts').Show()
			else:
				# Remove the old entry.
				del self.global_store.variables[self.var.name]

			self.var.name = var_new_name

		self.measurement_name_input.BackgroundColour = OK_BACKGROUND_COLOR
		self.set_title()

	def OnScaling(self, evt=None):
		def ok_callback(dlg):
			self.scaling_settings = dlg.GetValue()

		dlg = ScalingSettingsDialog(self, ok_callback)
		dlg.SetValue(self.scaling_settings)
		dlg.Show()

	def msg_resource(self, name, value=None):
		resource_name = self.var.resource_name

		if name == resource_name:
			self.resource = value

			if value is not None and not self.unwrapping:
				self.wrap_with_scaling(resource_name, value)



from unittest import main

from ... import awg5014b
from .. import mock_awg5014b

from ...tests.server.test_awg5014b import AWG5014BTest


# Don't lose the real device.
real_AWG5014B = awg5014b.AWG5014B
is_mock = AWG5014BTest.mock


def setup():
	# Run the tests with a fake device.
	awg5014b.AWG5014B = mock_awg5014b.MockAWG5014B
	AWG5014BTest.mock = True

def teardown():
	# Restore the real device for any remaining tests.
	awg5014b.AWG5014B = real_AWG5014B
	AWG5014BTest.mock = is_mock


if __name__ == '__main__':
	main()



import wx

from spacq.tool.box import triples_to_mesh

from ....tool.box import MessageDialog
from ..colormapped import ColormappedPlot
from .common.plot_setup import PlotSetupDialog


class ColormappedPlotPanel(wx.Panel):
	def __init__(self, parent, color_data, x_bounds, y_bounds, x_label, y_label,
			*args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		# Panel.
		panel_box = wx.BoxSizer(wx.VERTICAL)

		## Plot.
		self.plot = ColormappedPlot(self, x_bounds, y_bounds)
		panel_box.Add(self.plot.control, proportion=1, flag=wx.EXPAND)

		self.SetSizer(panel_box)

		self.plot.x_label, self.plot.y_label = x_label, y_label
		self.plot.color_data = color_data


class ColormappedPlotFrame(wx.Frame):
	bounds_format = '{0:.4e}'

	def __init__(self, parent, color_data, x_bounds, y_bounds, x_label, y_label,
			*args, **kwargs):
		wx.Frame.__init__(self, parent, *args, **kwargs)

		# Frame.
		frame_box = wx.BoxSizer(wx.VERTICAL)

		## Plot panel.
		self.panel = ColormappedPlotPanel(self, color_data, x_bounds, y_bounds,
				x_label, y_label)
		self.panel.SetMinSize((400, 300))
		frame_box.Add(self.panel, proportion=1, flag=wx.EXPAND)

		## Settings.
		settings_box = wx.BoxSizer(wx.HORIZONTAL)
		frame_box.Add(settings_box, flag=wx.CENTER)

		### Minimum value.
		settings_box.Add(wx.StaticText(self, label='Min: '),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.minimum_value_input = wx.TextCtrl(self,
				value=self.bounds_format.format(self.panel.plot.low_setting),
				size=(100, -1), style=wx.TE_PROCESS_ENTER)
		self.Bind(wx.EVT_TEXT_ENTER, self.OnMinValue, self.minimum_value_input)
		settings_box.Add(self.minimum_value_input, flag=wx.RIGHT, border=20)

		### Maximum value.
		settings_box.Add(wx.StaticText(self, label='Max: '),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.maximum_value_input = wx.TextCtrl(self,
				value=self.bounds_format.format(self.panel.plot.high_setting),
				size=(100, -1), style=wx.TE_PROCESS_ENTER)
		self.Bind(wx.EVT_TEXT_ENTER, self.OnMaxValue, self.maximum_value_input)
		settings_box.Add(self.maximum_value_input)

		self.SetSizerAndFit(frame_box)

	def OnMinValue(self, evt=None):
		value = self.minimum_value_input.Value
		try:
			value = float(value)
		except ValueError:
			value = 'auto'

		if value <= self.panel.plot.high_setting:
			self.panel.plot.low_setting = value

		# Update the text box.
		self.minimum_value_input.Value = self.bounds_format.format(self.panel.plot.low_setting)

	def OnMaxValue(self, evt=None):
		value = self.maximum_value_input.Value
		try:
			value = float(value)
		except ValueError:
			value = 'auto'

		if value >= self.panel.plot.low_setting:
			self.panel.plot.high_setting = value

		# Update the text box.
		self.maximum_value_input.Value = self.bounds_format.format(self.panel.plot.high_setting)


class ColormappedPlotSetupDialog(PlotSetupDialog):
	def __init__(self, parent, headings, data, *args, **kwargs):
		PlotSetupDialog.__init__(self, parent, headings, ['x', 'y', 'color'],
				*args, **kwargs)

		self.parent = parent
		self.headings = headings
		self.data = data

	def make_plot(self):
		try:
			x_data, y_data, z_data = [self.data[:,axis].astype(float) for axis in self.axes]
		except ValueError as e:
			MessageDialog(self, str(e), 'Invalid value').Show()
			return

		try:
			color_data, x_bounds, y_bounds, _ = triples_to_mesh(x_data, y_data, z_data)
		except Exception as e:
			MessageDialog(self, str(e), 'Conversion failure').Show()
			return

		x_label, y_label, z_label = [self.headings[x] for x in self.axes]
		title = '{0} vs ({1}, {2})'.format(z_label, x_label, y_label)

		frame = ColormappedPlotFrame(self.parent, color_data, x_bounds, y_bounds,
				x_label, y_label, title=title)
		frame.Show()

		return True



from chaco.api import VPlotContainer
from enable.api import Window
from math import floor, log10
from numpy import linspace
import wx

from spacq.interface.units import SIValues

from .plot.two_dimensional import TwoDimensionalPlot


class WaveformDisplay(TwoDimensionalPlot):
	marker_height = 50

	def __init__(self, parent, *args, **kwargs):
		TwoDimensionalPlot.__init__(self, parent, *args, **kwargs)

		self.padding_left = 50

		self.title = 'Waveform'

		self.vplot_container = VPlotContainer(use_backbuffer=True)
		self.vplot_container.stack_order = 'top_to_bottom'
		self.vplot_container.add(self)

	@property
	def control(self):
		return Window(self.parent, component=self.vplot_container).control

	def add_marker(self, num, data):
		marker_plot = TwoDimensionalPlot(self, height=self.marker_height, resizable='h')
		marker_plot.padding_left = self.padding_left

		marker_plot.x_data = self.x_data
		marker_plot.y_data = data
		marker_plot.title = 'Marker {0}'.format(num)

		# Synchronize with waveform plot.
		marker_plot.index_range = self.index_range

		self.vplot_container.add(marker_plot)


class WaveformPanel(wx.Panel):
	def __init__(self, parent, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		# Panel.
		panel_box = wx.BoxSizer(wx.VERTICAL)

		## Waveform plot.
		self.waveform_plot = WaveformDisplay(self)
		panel_box.Add(self.waveform_plot.control, proportion=1, flag=wx.EXPAND)

		self.SetSizer(panel_box)

	def SetValue(self, waveform, marker_data, frequency):
		max_time = len(waveform) / frequency.value
		# Find the order of magnitude (to within 3 orders, to keep it at n, u, m, etc).
		magnitude = 3 * floor(floor(log10(max_time)) / 3)

		self.waveform_plot.x_data = linspace(0, max_time / (10 ** magnitude), len(waveform))
		self.waveform_plot.y_data = waveform

		self.waveform_plot.x_label = '{0}s'.format(SIValues.prefixes_[magnitude])

		for num, data in marker_data.items():
			self.waveform_plot.add_marker(num, data)

		self.waveform_plot.x_autoscale()
		self.waveform_plot.y_autoscale()


class WaveformFrame(wx.Frame):
	def __init__(self, parent, output_name, *args, **kwargs):
		wx.Frame.__init__(self, parent, title=output_name, *args, **kwargs)

		# Frame.
		frame_box = wx.BoxSizer(wx.VERTICAL)

		self.panel = WaveformPanel(self)
		self.panel.SetMinSize((600, 400))
		frame_box.Add(self.panel, proportion=1, flag=wx.EXPAND)

		self.SetSizerAndFit(frame_box)

	def SetValue(self, *args):
		self.panel.SetValue(*args)



from functools import partial
from threading import Thread
import wx
from wx.lib import filebrowsebutton
from wx.lib.scrolledpanel import ScrolledPanel

from spacq.interface.pulse.parser import PulseError, PulseSyntaxError
from spacq.interface.pulse.program import Program
from spacq.interface.resources import Resource
from spacq.interface.units import IncompatibleDimensions, Quantity

from ..display.waveform import WaveformFrame
from ..tool.box import OK_BACKGROUND_COLOR, determine_wildcard, MessageDialog


class FileBrowseButton(filebrowsebutton.FileBrowseButton):
	ChangeValue = filebrowsebutton.FileBrowseButton.SetValue

	def SetBackgroundColour(self, colour):
		self.textControl.SetBackgroundColour(colour)


def pos_int_converter(x):
	try:
		result = int(x)

		if result <= 0:
			raise ValueError()

		return result
	except ValueError:
		raise ValueError('Expected positive integer')

def quantity_converter(x, symbols='s', dimensions='time', non_negative=True):
	try:
		q = Quantity(x)
		q.assert_dimensions(symbols)
	except (IncompatibleDimensions, ValueError):
		raise ValueError('Expected {0} quantity'.format(dimensions))

	if non_negative and q.value < 0:
		raise ValueError('Expected non-negative quantity')

	return q


class ParameterPanel(ScrolledPanel):
	"""
	A generic panel to display parameters of a particular type.
	"""

	attributes = False
	hide_variables = False
	use_resource_labels = False

	spacer_height = 15

	def extract_variables(self, prog):
		"""
		By default, extract the variables which pertain to the current type.
		"""

		return [k for k, v in prog.variables.items() if v == self.type]

	def extract_parameters(self, prog):
		"""
		By default, extract the parameters which pertain to the current type.
		"""

		variables = self.extract_variables(prog)

		return sorted([item for item in prog.all_values for variable in variables if item[0] == variable])

	@property
	def num_cols(self):
		"""
		Number of columns per row.
		"""

		# Label and input field.
		cols = 2

		# Label includes attribute name.
		if self.attributes:
			cols += 1

		# Label excludes variable name.
		if self.hide_variables:
			cols -= 1

		# Also resource label input field.
		if self.use_resource_labels:
			cols += 1

		return cols

	@property
	def input_col(self):
		"""
		The 0-based position of the growable input column.
		"""

		if self.use_resource_labels:
			# Second-to-last column.
			return self.num_cols - 2
		else:
			# Last column.
			return self.num_cols - 1

	def get_value(self, parameter):
		"""
		Get the value of a parameter as a string, or raise KeyError if not available.
		"""

		return str(self.values[parameter])

	def get_resource_label(self, parameter):
		"""
		Get the resource label for a parameter, or empty string if not available.
		"""

		try:
			return self.resource_labels[parameter]
		except KeyError:
			return ''

	@property
	def posn(self):
		return (self.cur_row, self.cur_col)

	def add_headings(self):
		"""
		Add column headings.
		"""

		if self.use_resource_labels:
			# Default value.
			self.parameter_sizer.Add(wx.StaticText(self, label='Default value'), (self.cur_row, self.input_col),
					flag=wx.EXPAND)

			# Resource label.
			self.parameter_sizer.Add(wx.StaticText(self, label='Resource label'), (self.cur_row, self.input_col + 1),
					flag=wx.EXPAND)
		else:
			self.parameter_sizer.Add(wx.StaticText(self, label=''), (self.cur_row, 0),
					flag=wx.EXPAND)

		self.cur_row += 1

	def add_resource_label(self, parameter):
		"""
		Add a resource label input.
		"""

		resource_input = wx.TextCtrl(self, style=wx.TE_PROCESS_ENTER)
		self.Bind(wx.EVT_TEXT, partial(self.OnResourceChange, parameter), resource_input)
		self.Bind(wx.EVT_TEXT_ENTER, partial(self.OnResourceInput, parameter), resource_input)

		self.parameter_sizer.Add(resource_input, self.posn, flag=wx.EXPAND)
		self.cur_col += 1

		label = self.get_resource_label(parameter)
		resource_input.ChangeValue(label)
		resource_input.default_background_color = resource_input.BackgroundColour
		if label:
			resource_input.BackgroundColour = OK_BACKGROUND_COLOR

	def add_row(self, parameter, input_type='text', increment_row=True):
		"""
		Add a parameter to the sizer and display the value if it is available.
		"""

		self.cur_col = 0

		if not self.hide_variables:
			if self.last_variable == parameter[0]:
				label = ''
			else:
				label = parameter[0]
				self.last_variable = parameter[0]

			self.parameter_sizer.Add(wx.StaticText(self, label=label), self.posn,
					flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
			self.cur_col += 1

		if self.attributes:
			self.parameter_sizer.Add(wx.StaticText(self, label=parameter[1]), self.posn,
					flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_LEFT)
			self.cur_col += 1

		if input_type == 'text':
			input = wx.TextCtrl(self, style=wx.TE_PROCESS_ENTER)

			self.Bind(wx.EVT_TEXT, partial(self.OnChange, parameter), input)
			self.Bind(wx.EVT_TEXT_ENTER, partial(self.OnInput, parameter), input)
		elif input_type == 'file':
			input = FileBrowseButton(self, labelText='File:', changeCallback=partial(self.OnInput, parameter))
		else:
			raise ValueError('Unrecognized type "{0}"'.format(input_type))

		self.parameter_sizer.Add(input, self.posn, flag=wx.EXPAND)
		self.cur_col += 1

		input.default_background_color = input.BackgroundColour

		try:
			input.ChangeValue(self.get_value(parameter))
		except KeyError:
			# No default value set.
			pass
		else:
			input.SetBackgroundColour(OK_BACKGROUND_COLOR)

		if self.use_resource_labels:
			self.add_resource_label(parameter)

		if increment_row:
			self.cur_row += 1

	def converter(self, parameter, x):
		"""
		Identity.
		"""

		return x

	def __init__(self, parent, global_store, prog, *args, **kwargs):
		ScrolledPanel.__init__(self, parent, *args, **kwargs)

		self.global_store = global_store
		self.prog = prog

		self.values = prog.values
		self.resource_labels = prog.resource_labels
		self.resources = prog.resources

		self.last_variable = None
		self.cur_row, self.cur_col = 0, 0

		parameters = self.extract_parameters(prog)

		# Panel.
		self.parameter_sizer = wx.GridBagSizer(hgap=5)

		self.parameter_sizer.AddGrowableCol(self.input_col, 1)
		if self.use_resource_labels:
			self.parameter_sizer.AddGrowableCol(self.input_col + 1, 1)

		## Headings.
		self.add_headings()

		## Parameter inputs.
		for parameter in parameters:
			self.add_row(parameter)

		self.SetSizer(self.parameter_sizer)
		self.SetupScrolling()

	def set_value(self, parameter, value):
		self.values[parameter] = value

	def del_value(self, parameter):
		try:
			del self.values[parameter]
		except KeyError:
			pass

	def set_resource_label(self, parameter, value, resource):
		self.resource_labels[parameter] = value
		self.resources[parameter] = resource

	def del_resource_label(self, parameter):
		try:
			label = self.resource_labels[parameter]
		except KeyError:
			pass
		else:
			del self.resource_labels[parameter]
			del self.resources[parameter]
			del self.global_store.resources[label]

	def OnChange(self, parameter, evt):
		# Awaiting validation.
		self.del_value(parameter)

		evt.EventObject.BackgroundColour = evt.EventObject.default_background_color

	def OnInput(self, parameter, evt):
		try:
			value = self.converter(parameter, evt.String)
		except ValueError as e:
			MessageDialog(self, str(e), 'Invalid value').Show()

			return

		# Validated.
		self.set_value(parameter, value)

		evt.EventObject.BackgroundColour = OK_BACKGROUND_COLOR

	def OnResourceChange(self, parameter, evt):
		# Awaiting validation.
		self.del_resource_label(parameter)

		evt.EventObject.BackgroundColour = evt.EventObject.default_background_color

	def OnResourceInput(self, parameter, evt):
		label = evt.String

		try:
			# Do nothing if there has not been a change.
			if label == self.resource_labels[parameter]:
				return
		except KeyError:
			pass

		# The actual setter is generated when the program is cloned.
		resource = Resource(setter=lambda x: None)

		try:
			self.global_store.resources[label] = resource
		except KeyError as e:
			MessageDialog(self, str(e[0]), 'Resource label conflicts').Show()

			return

		# Validated.
		self.set_resource_label(parameter, label, resource)

		evt.EventObject.BackgroundColour = OK_BACKGROUND_COLOR


class AcqMarkerPanel(ParameterPanel):
	type = 'acq_marker'
	name = 'Acquisition'
	attributes = True
	hide_variables = True

	def converter(self, parameter, x):
		x = ParameterPanel.converter(self, parameter, x)

		if parameter[1] == 'marker_num':
			return pos_int_converter(x)
		elif parameter[1] == 'output':
			try:
				if self.prog.variables[x] == 'output':
					return x
				else:
					raise KeyError()
			except KeyError:
				raise ValueError('Expected valid output name')

	def __init__(self, *args, **kwargs):
		ParameterPanel.__init__(self, *args, **kwargs)

		# Spacer.
		self.parameter_sizer.Add((-1, self.spacer_height), (self.cur_row, 0))
		self.cur_row += 1

		# Times to average.
		self.parameter_sizer.Add(wx.StaticText(self, label='Times to average'), (self.cur_row, 0),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.times_average_input = wx.TextCtrl(self, style=wx.TE_PROCESS_ENTER)
		self.parameter_sizer.Add(self.times_average_input, (self.cur_row, 1), flag=wx.EXPAND)
		self.cur_row += 1

		self.times_average_input.Value = str(self.prog.times_average)
		self.times_average_input.default_background_color = self.times_average_input.BackgroundColour
		self.times_average_input.BackgroundColour = OK_BACKGROUND_COLOR

		self.Bind(wx.EVT_TEXT, self.OnTimesAverageChange, self.times_average_input)
		self.Bind(wx.EVT_TEXT_ENTER, self.OnTimesAverageInput, self.times_average_input)

		# Post-trigger delay.
		self.parameter_sizer.Add(wx.StaticText(self, label='Post-trigger delay'), (self.cur_row, 0),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.delay_input = wx.TextCtrl(self, style=wx.TE_PROCESS_ENTER)
		self.parameter_sizer.Add(self.delay_input, (self.cur_row, 1), flag=wx.EXPAND)
		self.cur_row += 1

		self.delay_input.Value = str(self.prog.acq_delay)
		self.delay_input.default_background_color = self.delay_input.BackgroundColour
		self.delay_input.BackgroundColour = OK_BACKGROUND_COLOR

		self.Bind(wx.EVT_TEXT, self.OnDelayChange, self.delay_input)
		self.Bind(wx.EVT_TEXT_ENTER, self.OnDelayInput, self.delay_input)

	def OnTimesAverageChange(self, evt=None):
		self.times_average_input.BackgroundColour = self.times_average_input.default_background_color

	def OnTimesAverageInput(self, evt=None):
		try:
			value = pos_int_converter(self.times_average_input.Value)
		except ValueError as e:
			MessageDialog(self, str(e), 'Invalid value').Show()

			return

		self.prog.times_average = value

		self.times_average_input.BackgroundColour = OK_BACKGROUND_COLOR

	def OnDelayChange(self, evt=None):
		self.delay_input.BackgroundColour = self.delay_input.default_background_color

	def OnDelayInput(self, evt=None):
		try:
			value = quantity_converter(self.delay_input.Value, 's', 'time')
		except ValueError as e:
			MessageDialog(self, str(e), 'Invalid value').Show()

			return

		self.prog.acq_delay = value

		self.delay_input.BackgroundColour = OK_BACKGROUND_COLOR


class DelayPanel(ParameterPanel):
	type = 'delay'
	name = 'Delays'
	use_resource_labels = True

	def converter(self, parameter, x):
		x = ParameterPanel.converter(self, parameter, x)

		return quantity_converter(x)


class IntPanel(ParameterPanel):
	type = 'int'
	name = 'Integers'
	use_resource_labels = True

	def converter(self, parameter, x):
		x = ParameterPanel.converter(self, parameter, x)

		try:
			return int(x)
		except ValueError:
			raise ValueError('Expected integer')


class OutputPanel(ParameterPanel):
	type = 'output'
	name = 'Outputs'

	def extract_parameters(self, prog):
		return sorted([(x,) for x in self.extract_variables(prog)])

	@property
	def num_cols(self):
		return ParameterPanel.num_cols.__get__(self) + 1

	@property
	def input_col(self):
		return self.num_cols - 2

	def get_value(self, parameter):
		result = self.prog.output_channels[parameter[0]]

		if result is not None:
			return str(result)
		else:
			raise KeyError(parameter[0])

	def add_row(self, parameter):
		ParameterPanel.add_row(self, parameter, increment_row=False)

		view_button = wx.Button(self, label='View')
		self.parameter_sizer.Add(view_button, self.posn)
		self.cur_col += 1
		self.Bind(wx.EVT_BUTTON, partial(self.OnView, parameter), view_button)

		self.cur_row += 1

	def converter(self, parameter, x):
		if x == '':
			return x
		else:
			return pos_int_converter(x)

	def __init__(self, *args, **kwargs):
		ParameterPanel.__init__(self, *args, **kwargs)

		# Spacer.
		self.parameter_sizer.Add((-1, self.spacer_height), (self.cur_row, 0))
		self.cur_row += 1

		# Frequency input.
		self.parameter_sizer.Add(wx.StaticText(self, label='Sampling rate'), (self.cur_row, 0),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.freq_input = wx.TextCtrl(self, style=wx.TE_PROCESS_ENTER)
		self.parameter_sizer.Add(self.freq_input, (self.cur_row, 1), flag=wx.EXPAND)
		self.cur_row += 1

		self.freq_input.Value = str(self.prog.frequency)
		self.freq_input.default_background_color = self.freq_input.BackgroundColour
		self.freq_input.BackgroundColour = OK_BACKGROUND_COLOR

		self.Bind(wx.EVT_TEXT, self.OnFrequencyChange, self.freq_input)
		self.Bind(wx.EVT_TEXT_ENTER, self.OnFrequencyInput, self.freq_input)

		# Spacer.
		self.parameter_sizer.Add((-1, self.spacer_height), (self.cur_row, 0))
		self.cur_row += 1

		# AWG input.
		self.parameter_sizer.Add(wx.StaticText(self, label='AWG'), (self.cur_row, 0),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.awg_input = wx.TextCtrl(self, style=wx.TE_PROCESS_ENTER)
		self.parameter_sizer.Add(self.awg_input, (self.cur_row, 1), flag=wx.EXPAND)
		self.cur_row += 1

		self.awg_input.Value = self.prog.awg
		self.awg_input.default_background_color = self.awg_input.BackgroundColour
		self.awg_input.BackgroundColour = OK_BACKGROUND_COLOR

		self.Bind(wx.EVT_TEXT, self.OnAWGChange, self.awg_input)
		self.Bind(wx.EVT_TEXT_ENTER, self.OnAWGInput, self.awg_input)

		# Oscilloscope input.
		self.parameter_sizer.Add(wx.StaticText(self, label='Oscilloscope'), (self.cur_row, 0),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.oscilloscope_input = wx.TextCtrl(self, style=wx.TE_PROCESS_ENTER)
		self.parameter_sizer.Add(self.oscilloscope_input, (self.cur_row, 1), flag=wx.EXPAND)
		self.cur_row += 1

		self.oscilloscope_input.Value = self.prog.oscilloscope
		self.oscilloscope_input.default_background_color = self.oscilloscope_input.BackgroundColour
		self.oscilloscope_input.BackgroundColour = OK_BACKGROUND_COLOR

		self.Bind(wx.EVT_TEXT, self.OnOscilloscopeChange, self.oscilloscope_input)
		self.Bind(wx.EVT_TEXT_ENTER, self.OnOscilloscopeInput, self.oscilloscope_input)

	def set_value(self, parameter, value):
		if value == '':
			value = None

		self.prog.output_channels[parameter[0]] = value

	def OnFrequencyChange(self, evt=None):
		self.freq_input.BackgroundColour = self.freq_input.default_background_color

	def OnFrequencyInput(self, evt=None):
		try:
			value = quantity_converter(self.freq_input.Value, 'Hz', 'frequency')
		except ValueError as e:
			MessageDialog(self, str(e), 'Invalid value').Show()

			return

		self.prog.frequency = value

		self.freq_input.BackgroundColour = OK_BACKGROUND_COLOR

	def OnAWGChange(self, evt=None):
		self.awg_input.BackgroundColour = self.awg_input.default_background_color

	def OnAWGInput(self, evt=None):
		self.prog.awg = self.awg_input.Value

		self.awg_input.BackgroundColour = OK_BACKGROUND_COLOR

	def OnOscilloscopeChange(self, evt=None):
		self.oscilloscope_input.BackgroundColour = self.oscilloscope_input.default_background_color

	def OnOscilloscopeInput(self, evt=None):
		self.prog.oscilloscope = self.oscilloscope_input.Value

		self.oscilloscope_input.BackgroundColour = OK_BACKGROUND_COLOR

	def OnView(self, parameter, evt=None):
		def show_frame(waveform, markers, frequency):
			view_frame = WaveformFrame(self, parameter[0])
			view_frame.SetValue(waveform, markers, frequency)
			view_frame.Show()

		def show_error(error, monospace=False):
			MessageDialog(self, error, 'Waveform generation error', monospace=monospace).Show()

		def show_waveform():
			try:
				waveforms = self.prog.generate_waveforms()
			except ValueError as e:
				wx.CallAfter(show_error, str(e))

				return
			except PulseError as e:
				wx.CallAfter(show_error, '\n'.join((e[0])), monospace=True)

				return

			waveform, markers = waveforms[parameter[0]]

			wx.CallAfter(show_frame, waveform, markers, self.prog.frequency)

		thr = Thread(target=show_waveform)
		thr.daemon = True
		thr.start()


class PulsePanel(ParameterPanel):
	type = 'pulse'
	name = 'Pulses'
	attributes = True
	use_resource_labels = True

	def add_resource_label(self, parameter):
		if parameter[1] == 'shape':
			self.cur_col += 1
		else:
			ParameterPanel.add_resource_label(self, parameter)

	def add_row(self, parameter):
		kwargs = {}
		if parameter[1] == 'shape':
			kwargs['input_type'] = 'file'

		return ParameterPanel.add_row(self, parameter, **kwargs)

	def converter(self, parameter, x):
		x = ParameterPanel.converter(self, parameter, x)

		if parameter[1] == 'amplitude':
			return quantity_converter(x, 'V', 'voltage', False)
		elif parameter[1] == 'length':
			return quantity_converter(x)
		elif parameter[1] == 'shape':
			return x


class PulseProgramPanel(wx.Panel):
	"""
	A panel to display and change all the parameters of a program.
	"""

	panel_types = {'acq_marker': AcqMarkerPanel, 'delay': DelayPanel, 'int': IntPanel,
			'output': OutputPanel, 'pulse': PulsePanel}

	def __init__(self, parent, global_store, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		self.global_store = global_store

		self.prog = None

		# Panel.
		panel_box = wx.BoxSizer(wx.VERTICAL)

		## Notebook.
		self.parameter_notebook = wx.Notebook(self)
		self.parameter_notebook.SetMinSize((600, 400))
		panel_box.Add(self.parameter_notebook, proportion=1, flag=wx.EXPAND|wx.ALL, border=5)

		self.SetSizerAndFit(panel_box)

	def create_parameter_panels(self, prog):
		types = set(prog.variables.values())

		for type in sorted(types):
			try:
				result = self.panel_types[type](self.parameter_notebook, self.global_store, prog)
			except KeyError:
				MessageDialog('Unrecognized variable type "{0}"'.format(type)).Show()

				return

			self.parameter_notebook.AddPage(result, result.name)

	def OnOpen(self, prog):
		self.prog = prog

		self.create_parameter_panels(self.prog)

	def OnClose(self):
		self.prog = None

		self.parameter_notebook.DeleteAllPages()


class PulseProgramFrame(wx.Frame):
	default_title = 'Pulse program'

	def __init__(self, parent, global_store, close_callback, *args, **kwargs):
		if 'title' not in kwargs:
			kwargs['title'] = self.default_title
		else:
			self.default_title = kwargs['title']

		wx.Frame.__init__(self, parent, *args, **kwargs)

		self.global_store = global_store
		self.close_callback = close_callback

		# Menu.
		menuBar = wx.MenuBar()

		## File.
		menu = wx.Menu()
		menuBar.Append(menu, '&File')

		item = menu.Append(wx.ID_OPEN, '&Open...')
		self.Bind(wx.EVT_MENU, self.OnMenuFileOpen, item)

		item = menu.Append(wx.ID_CLOSE, '&Close')
		self.Bind(wx.EVT_MENU, self.OnMenuFileClose, item)

		self.SetMenuBar(menuBar)

		# Frame.
		frame_box = wx.BoxSizer(wx.VERTICAL)

		self.pulse_panel = PulseProgramPanel(self, self.global_store)
		frame_box.Add(self.pulse_panel, proportion=1, flag=wx.EXPAND)

		self.SetSizerAndFit(frame_box)

		self.Bind(wx.EVT_CLOSE, self.OnClose)

		# Reload existing program.
		if self.global_store.pulse_program is not None:
			self.load_program(self.global_store.pulse_program)

	def load_program(self, prog):
			self.Title = '{0} - {1}'.format(prog.filename, self.default_title)

			self.pulse_panel.OnOpen(prog)

	def OnMenuFileOpen(self, evt=None):
		wildcard = determine_wildcard('pulse', 'Pulse program')
		dlg = wx.FileDialog(parent=self, message='Load...', wildcard=wildcard,
				style=wx.FD_OPEN)

		if dlg.ShowModal() == wx.ID_OK:
			path = dlg.GetPath()

			try:
				prog = Program.from_file(path)
			except PulseSyntaxError as e:
				MessageDialog(self, '\n'.join(e[0]), 'Compilation error', monospace=True).Show()

				return

			# Only purge the previous file if this one has been opened successfully.
			self.OnMenuFileClose()

			self.load_program(prog)

			self.global_store.pulse_program = prog

	def OnMenuFileClose(self, evt=None):
		if self.global_store.pulse_program is None:
			return

		self.pulse_panel.OnClose()

		self.Title = self.default_title

		for label in self.global_store.pulse_program.resource_labels.values():
			del self.global_store.resources[label]

		self.global_store.pulse_program = None

	def OnClose(self, evt):
		self.close_callback()

		evt.Skip()



from functools import partial
from pubsub import pub
from threading import Thread
from time import sleep
import wx
from wx.lib.agw.floatspin import FloatSpin

from spacq.gui.tool.box import load_csv, save_csv, Dialog, MessageDialog
from spacq.interface.units import Quantity

"""
Configuration for a VoltageSource.
"""


class VoltageSourceTunerDialog(Dialog):
	"""
	A dialog for tuning a voltage source port.
	"""

	def __init__(self, parent, global_store, ok_callback, port,	*args, **kwargs):
		Dialog.__init__(self, parent, title='Port {0} tuning'.format(port.num))

		self.global_store = global_store
		self.ok_callback = ok_callback
		self.port = port

		# Dialog.
		dialog_box = wx.BoxSizer(wx.VERTICAL)

		## Self-calibration.
		calibration_static_box = wx.StaticBox(self, label='DAC self-calibration')
		calibration_box = wx.StaticBoxSizer(calibration_static_box, wx.VERTICAL)
		dialog_box.Add(calibration_box, flag=wx.EXPAND|wx.ALL, border=5)

		self.calibrate_button = wx.Button(self, label='Self-calibrate')
		self.Bind(wx.EVT_BUTTON, self.OnCalibrate, self.calibrate_button)
		calibration_box.Add(self.calibrate_button, flag=wx.EXPAND)

		## Tuning.
		tuning_static_box = wx.StaticBox(self, label='Tuning')
		tuning_box = wx.StaticBoxSizer(tuning_static_box, wx.VERTICAL)
		dialog_box.Add(tuning_box, flag=wx.EXPAND)

		### Autotune.
		autotuning_static_box = wx.StaticBox(self, label='Autotuning')
		autotuning_box = wx.StaticBoxSizer(autotuning_static_box, wx.VERTICAL)
		tuning_box.Add(autotuning_box, flag=wx.EXPAND|wx.ALL, border=5)

		autotuning_sizer = wx.FlexGridSizer(rows=3, cols=2, hgap=5)
		autotuning_box.Add(autotuning_sizer, flag=wx.CENTER)

		autotuning_sizer.Add(wx.StaticText(self, label='Resource name:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.resource_name_input = wx.TextCtrl(self, size=(300,-1))
		autotuning_sizer.Add(self.resource_name_input)

		autotuning_sizer.Add(wx.StaticText(self, label='Max:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.automax_input = FloatSpin(self, value=1, min_val=-10, max_val=10, increment=1,
				digits=5)
		autotuning_sizer.Add(self.automax_input)

		autotuning_sizer.Add(wx.StaticText(self, label='Min:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.automin_input = FloatSpin(self, value=-1, min_val=-10, max_val=10, increment=1,
				digits=5)
		autotuning_sizer.Add(self.automin_input)

		self.autotune_button = wx.Button(self, label='Autotune')
		self.Bind(wx.EVT_BUTTON, self.OnAutotune, self.autotune_button)
		autotuning_box.Add(self.autotune_button, flag=wx.EXPAND)

		### Manual tune.
		tuning_sizer = wx.FlexGridSizer(rows=2, cols=2, hgap=5)
		tuning_box.Add(tuning_sizer, flag=wx.CENTER)

		tuning_sizer.Add(wx.StaticText(self, label='Gain:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.gain_input = FloatSpin(self, value=0, min_val=-1e6, max_val=1e6, increment=1,
				digits=5)
		tuning_sizer.Add(self.gain_input)

		tuning_sizer.Add(wx.StaticText(self, label='Offset:'),
				flag=wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT)
		self.offset_input = FloatSpin(self, value=0, min_val=-1e6, max_val=1e6, increment=1,
				digits=5)
		tuning_sizer.Add(self.offset_input)

		## End buttons.
		button_box = wx.BoxSizer(wx.HORIZONTAL)
		dialog_box.Add(button_box, flag=wx.CENTER|wx.ALL, border=5)

		ok_button = wx.Button(self, wx.ID_OK)
		self.Bind(wx.EVT_BUTTON, self.OnOk, ok_button)
		button_box.Add(ok_button)

		cancel_button = wx.Button(self, wx.ID_CANCEL)
		button_box.Add(cancel_button)

		self.SetSizerAndFit(dialog_box)

	def autotune(self, resource):
		gain, offset = self.port.autotune(resource, set_result=False,
				min_value=self.automin_input.GetValue(),
				max_value=self.automax_input.GetValue())

		wx.CallAfter(self.gain_input.SetValue, gain)
		wx.CallAfter(self.offset_input.SetValue, offset)
		wx.CallAfter(self.autotune_button.Enable)

	def self_calbrate(self):
		self.port.apply_settings(calibrate=True)

		sleep(self.port.calibration_delay)
		wx.CallAfter(self.calibrate_button.Enable)

	def SetValue(self, gain, offset):
		self.gain_input.SetValue(gain)
		self.offset_input.SetValue(offset)

	def GetValue(self):
		return (self.gain_input.GetValue(), self.offset_input.GetValue())

	def OnAutotune(self, evt=None):
		name = self.resource_name_input.Value

		if not name:
			MessageDialog(self, 'No resource provided').Show()
			return

		try:
			resource = self.global_store.resources[name]
		except KeyError:
			MessageDialog(self, name, 'Missing resource').Show()
			return

		if not resource.readable:
			MessageDialog(self, name, 'Unreadable resource').Show()
			return

		self.autotune_button.Disable()

		thr = Thread(target=self.autotune, args=(resource,))
		thr.daemon = True
		thr.start()

	def OnCalibrate(self, evt=None):
		self.calibrate_button.Disable()

		thr = Thread(target=self.self_calbrate)
		thr.daemon = True
		thr.start()

	def OnOk(self, evt=None):
		self.ok_callback(self)

		self.Destroy()


class VoltageSourceSettingsPanel(wx.Panel):
	"""
	All the settings for a voltage source.
	"""

	def __init__(self, parent, global_store, vsrc, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		self.global_store = global_store
		self.vsrc = vsrc

		self.port_value_inputs = []
		self.port_buttons = []

		# Panel.
		panel_box = wx.BoxSizer(wx.VERTICAL)

		## Ports.
		ports_box = wx.FlexGridSizer(rows=8, cols=2)
		panel_box.Add(ports_box)

		for port in xrange(16):
			port_static_box = wx.StaticBox(self, label='Port {0} '.format(port))
			port_box = wx.StaticBoxSizer(port_static_box, wx.HORIZONTAL)
			ports_box.Add(port_box, flag=wx.ALL, border=5)

			spin = FloatSpin(self, value=0, min_val=-10, max_val=10, increment=1, digits=6)
			self.port_value_inputs.append(spin)
			port_box.Add(spin)

			port_box.Add(wx.StaticText(self, label='V'))

			set_button = wx.Button(self, label='Set', style=wx.BU_EXACTFIT)
			set_button.Bind(wx.EVT_BUTTON, partial(self.OnSetVoltage, port))
			port_box.Add(set_button)

			tune_button = wx.Button(self, label='Tune...', style=wx.BU_EXACTFIT)
			tune_button.Bind(wx.EVT_BUTTON, partial(self.OnTune, port))
			port_box.Add(tune_button)

			self.port_buttons.append((set_button, tune_button))

		## All ports.
		button_static_box = wx.StaticBox(self, label='All ports')
		button_box = wx.StaticBoxSizer(button_static_box, wx.HORIZONTAL)
		panel_box.Add(button_box, flag=wx.CENTER)

		### Zero.
		zero_all_button = wx.Button(self, label='Zero')
		self.Bind(wx.EVT_BUTTON, self.OnZeroAll, zero_all_button)
		button_box.Add(zero_all_button, flag=wx.CENTER)

		### Self-calibrate.
		self.calibrate_all_button = wx.Button(self, label='Self-calibrate')
		self.Bind(wx.EVT_BUTTON, self.OnCalibrateAll, self.calibrate_all_button)
		button_box.Add(self.calibrate_all_button, flag=wx.CENTER)

		### Load tuning.
		tuning_data_static_box = wx.StaticBox(self, label='Tuning data')
		tuning_data_box = wx.StaticBoxSizer(tuning_data_static_box, wx.HORIZONTAL)
		button_box.Add(tuning_data_box)

		#### Save.
		tuning_data_save_button = wx.Button(self, label='Save...')
		self.Bind(wx.EVT_BUTTON, self.OnSave, tuning_data_save_button)
		tuning_data_box.Add(tuning_data_save_button)

		#### Load.
		tuning_data_load_button = wx.Button(self, label='Load...')
		self.Bind(wx.EVT_BUTTON, self.OnLoad, tuning_data_load_button)
		tuning_data_box.Add(tuning_data_load_button)

		self.SetSizer(panel_box)

	def self_calbrate_all(self):
		delay = 0 # s

		for port in self.vsrc.ports:
			# Use the largest delay.
			if port.calibration_delay > delay:
				delay = port.calibration_delay

			port.apply_settings(calibrate=True)

		sleep(delay)
		wx.CallAfter(self.calibrate_all_button.Enable)

	def zero_all(self):
		for port in self.vsrc.ports:
			port.voltage = Quantity(0.0, 'V')

	def OnSetVoltage(self, port_num, evt=None):
		try:
			self.vsrc.ports[port_num].voltage = Quantity(self.port_value_inputs[port_num].GetValue(), 'V')
		except ValueError as e:
			MessageDialog(self, str(e), 'Invalid value').Show()

	def OnTune(self, port_num, evt=None):
		port = self.vsrc.ports[port_num]

		def ok_callback(dlg):
			port.gain, port.offset = dlg.GetValue()

		dlg = VoltageSourceTunerDialog(self, self.global_store, ok_callback, port)
		dlg.SetValue(port.gain, port.offset)
		dlg.Show()

	def OnCalibrateAll(self, evt=None):
		self.calibrate_all_button.Disable()

		thr = Thread(target=self.self_calbrate_all)
		thr.daemon = True
		thr.start()

	def OnZeroAll(self, evt=None):
		thr = Thread(target=self.zero_all)
		thr.daemon = True
		thr.start()

	def OnSave(self, evt=None):
		values = [[port.gain, port.offset] for port in self.vsrc.ports]

		try:
			save_csv(self, values)
		except IOError as e:
			MessageDialog(self, str(e), 'Save error').Show()
			return

	def OnLoad(self, evt=None):
		try:
			result = load_csv(self)

			if result is None:
				return

			has_header, values, _ = result

			if has_header:
				port_values = values[1:]
			else:
				port_values = values

			if len(port_values) != len(self.vsrc.ports):
				raise ValueError('Invalid number of ports.')

			for i, port_value in enumerate(port_values):
				if len(port_value) != 2:
					raise ValueError('Invalid number of settings for port {0}.'.format(i))

				try:
					float(port_value[0])
					float(port_value[1])
				except TypeError:
					raise ValueError('Not a number for port {0}.'.format(i))
		except (IOError, ValueError) as e:
			MessageDialog(self, str(e), 'Load error').Show()
			return

		for port, values in zip(self.vsrc.ports, port_values):
			port.gain = float(values[0])
			port.offset = float(values[1])


class VoltageSourceSettingsDialog(Dialog):
	"""
	A wrapper for VoltageSourceSettingsPanel.
	"""

	def __init__(self, parent, global_store, vsrc_name, *args, **kwargs):
		# If the device doesn't exist, give up.
		try:
			vsrc = global_store.devices[vsrc_name].device
		except (KeyError, AttributeError):
			self.Destroy()

			return

		Dialog.__init__(self, parent, title='Voltage source settings', *args, **kwargs)

		self.vsrc_name = vsrc_name

		# Dialog.
		dialog_box = wx.BoxSizer(wx.VERTICAL)

		## Settings panel.
		self.panel = VoltageSourceSettingsPanel(self, global_store, vsrc)
		dialog_box.Add(self.panel)

		self.SetSizerAndFit(dialog_box)

		# Subscriptions.
		pub.subscribe(self.msg_device, 'device.added')
		pub.subscribe(self.msg_device, 'device.removed')

	def msg_device(self, name, value=None):
		if name == self.vsrc_name:
			# Device has changed, so we can't trust it anymore.
			self.Destroy()

			return



#!/usr/bin/env python2

import logging
logging.basicConfig(level=logging.WARNING)

from functools import partial
import wx

from spacq import VERSION
from spacq.gui.display.plot.static.delegator import formats, available_formats
from spacq.gui.display.table.filter import FilterListDialog
from spacq.gui.display.table.generic import TabularDisplayFrame
from spacq.gui.tool.box import load_csv, MessageDialog


class DataExplorerApp(wx.App):
	default_title = 'Data Explorer'

	def OnInit(self):
		self.filters = {}
		self.filter_columns = {}
		self.filter_dialog = None

		# Frames.
		self.csv_frame = TabularDisplayFrame(None, title=self.default_title)

		# Menu.
		menuBar = wx.MenuBar()

		## File.
		menu = wx.Menu()
		menuBar.Append(menu, '&File')

		item = menu.Append(wx.ID_OPEN, '&Open...')
		self.Bind(wx.EVT_MENU, self.OnMenuFileOpen, item)

		item = menu.Append(wx.ID_CLOSE, '&Close')
		self.Bind(wx.EVT_MENU, self.OnMenuFileClose, item)

		menu.AppendSeparator()

		self.filter_menu_item = menu.Append(wx.ID_ANY, '&Filters...')
		self.filter_menu_item.Enable(False)
		self.Bind(wx.EVT_MENU, self.OnMenuFileFilters, self.filter_menu_item)

		menu.AppendSeparator()

		item = menu.Append(wx.ID_EXIT, 'E&xit')
		self.Bind(wx.EVT_MENU, self.OnMenuFileExit, item)

		## Plot.
		menu = wx.Menu()
		menuBar.Append(menu, '&Plot')

		menu.Append(wx.ID_ANY, ' 2D:').Enable(False)

		self.two_dimensional_menu = menu.Append(wx.ID_ANY, '&Curve...')
		self.Bind(wx.EVT_MENU, partial(self.create_plot, formats.two_dimensional),
				self.two_dimensional_menu)

		menu.AppendSeparator()

		menu.Append(wx.ID_ANY, ' 3D:').Enable(False)

		self.colormapped_menu = menu.Append(wx.ID_ANY, '&Colormapped...')
		self.Bind(wx.EVT_MENU, partial(self.create_plot, formats.colormapped),
				self.colormapped_menu)

		self.surface_menu = menu.Append(wx.ID_ANY, '&Surface...')
		self.Bind(wx.EVT_MENU, partial(self.create_plot, formats.surface),
				self.surface_menu)

		menu.AppendSeparator()

		menu.Append(wx.ID_ANY, ' List:').Enable(False)

		self.waveforms_menu = menu.Append(wx.ID_ANY, '&Waveforms...')
		self.Bind(wx.EVT_MENU, partial(self.create_plot, formats.waveforms, type='list'),
				self.waveforms_menu)

		## Help.
		menu = wx.Menu()
		menuBar.Append(menu, '&Help')

		### About.
		item = menu.Append(wx.ID_ABOUT, '&About...')
		self.Bind(wx.EVT_MENU, self.OnMenuHelpAbout, item)

		self.csv_frame.SetMenuBar(menuBar)

		self.update_plot_menus(False)

		# Display.
		self.csv_frame.Show()
		self.csv_frame.SetSize((800, 600))

		self.SetTopWindow(self.csv_frame)
		self.csv_frame.Raise()

		return True

	def update_plot_menus(self, status):
		"""
		If status is True, enable the plot menus corresponding to the available formats. Otherwise, disable all.
		"""

		pairs = [
			(formats.two_dimensional, self.two_dimensional_menu),
			(formats.colormapped, self.colormapped_menu),
			(formats.surface, self.surface_menu),
			(formats.waveforms, self.waveforms_menu),
		]

		for format, menu in pairs:
			if not status or format in available_formats:
				menu.Enable(status)

	def create_plot(self, format, evt=None, type='scalar'):
		"""
		Open up a dialog to configure the selected plot format.
		"""

		headings, rows, types = self.csv_frame.display_panel.GetValue(types=[type])
		available_formats[format](self.csv_frame, headings, rows).Show()

	def OnMenuFileOpen(self, evt=None):
		try:
			result = load_csv(self.csv_frame)
		except IOError as e:
			MessageDialog(self.csv_frame, str(e), 'Could not load data').Show()
			return

		if result is None:
			return
		else:
			self.OnMenuFileClose()

		has_header, values, filename = result
		self.csv_frame.display_panel.from_csv_data(has_header, values)
		self.csv_frame.Title = '{0} - {1}'.format(filename, self.default_title)

		self.update_plot_menus(len(self.csv_frame.display_panel) > 0)

		self.filter_menu_item.Enable(True)

	def OnMenuFileClose(self, evt=None):
		self.csv_frame.display_panel.SetValue([], [])
		self.csv_frame.Title = self.default_title

		self.update_plot_menus(False)

		self.filter_menu_item.Enable(False)

		if self.filter_dialog is not None:
			self.filter_dialog.Close()

		self.filters = {}
		self.filter_columns = {}

	def OnMenuFileFilters(self, evt=None):
		def close_callback(dlg):
			self.filters = dlg.filters
			self.filter_columns = dlg.filter_columns

			self.filter_dialog = None

		if self.filter_dialog is None:
			self.filter_dialog = FilterListDialog(self.csv_frame, self.csv_frame.display_panel.table,
					close_callback, self.filters, self.filter_columns)
			self.filter_dialog.Show()

		self.filter_dialog.Raise()

	def OnMenuFileExit(self, evt=None):
		if self.csv_frame:
			self.csv_frame.Close()

	def OnMenuHelpAbout(self, evt=None):
		info = wx.AboutDialogInfo()
		info.SetName('Data Explorer')
		info.SetDescription('An application for displaying data in tabular and graphical form.\n'
			'\n'
			'Using Spanish Acquisition version {0}.'.format(VERSION)
		)

		wx.AboutBox(info)


if __name__ == "__main__":
	app = DataExplorerApp()
	app.MainLoop()



import logging
log = logging.getLogger(__name__)

from copy import deepcopy
from math import log10
from numpy import allclose
import quantities as pq

"""
Tools for working with quantities and units.
"""


class IncompatibleDimensions(TypeError):
	"""
	Operation on quantities with different dimensions.
	"""

	pass


class SIValues(object):
	"""
	Values for SI units.
	"""

	prefixes = {
		'y': -24,
		'z': -21,
		'a': -18,
		'f': -15,
		'p': -12,
		'n': -9,
		'u': -6,
		'm': -3,
		'c': -2,
		'd': -1,
		'': 0,
		'da': 1,
		'h': 2,
		'k': 3,
		'M': 6,
		'G': 9,
		'T': 12,
		'P': 15,
		'E': 18,
		'Z': 21,
		'Y': 24,
	}
	prefixes_ = dict([(v, k) for (k, v) in prefixes.items()])

	# SI base units. Note the g instead of kg.
	units = set(['A', 'cd', 'g', 'K', 'm', 'mol', 's'])
	# SI derived units. Add more as necessary.
	units.update(['Hz', 'J', 'N', 'T', 'V'])


class Quantity(object):
	"""
	A quantity with a value and dimensions.
	"""

	@staticmethod
	def parse_units(string):
		"""
		Convert group of unit symbols to pq-acceptable notation:
			Determine the multipliers and strip the prefixes.
			Use "*" and "**" to combine units.

		eg. ' mN.m.ks-2' -> 'N*m*s**-2', 3
		"""

		symbols = [x.strip() for x in string.split('.')]

		result_units = []
		result_multiplier = 0

		for symbol in symbols:
			symbol_unit, symbol_multiplier = None, None
			for prefix, multiplier in SIValues.prefixes.items():
				if not symbol.startswith(prefix):
					continue

				unit = symbol[len(prefix):]

				exponent = None
				for i in xrange(len(unit)):
					try:
						exponent = float(unit[i:])
						break
					except ValueError:
						pass

				if exponent is not None:
					unit = unit[:i]

				if unit not in SIValues.units:
					continue

				if symbol_unit is None:
					if exponent is not None:
						unit += '**{0}'.format(exponent)
					symbol_unit, symbol_multiplier = unit, multiplier
				else:
					# Already found a match.
					raise ValueError('Ambiguous unit symbol: "{0}"'.format(symbol))

			if symbol_unit is not None:
				result_units.append(symbol_unit)
				result_multiplier += symbol_multiplier
			else:
				# Did not find anything.
				raise ValueError('Unrecognized unit symbol: "{0}"'.format(symbol))

		return ('*'.join(result_units), result_multiplier)

	@staticmethod
	def from_string(string):
		"""
		Separate the value from the units.
		"""

		for i in xrange(len(string), 0, -1):
			try:
				value = float(string[:i])
			except ValueError:
				continue

			return value, string[i:]

		raise ValueError(string)

	def __init__(self, value, units=None):
		"""
		Both ('100 ms') and (100, 'ms') are acceptable.
		"""

		if isinstance(value, basestring):
			value, units = self.from_string(value)

		# Always work with single floats.
		value = float(value)

		log.debug('Creating Quantity: {0}, {1}'.format(value, units))

		# Remove unit prefixes.
		new_units, multiplier = self.parse_units(units)
		new_value = value * (10 ** multiplier)

		# Normalize to SI base units.
		original_quantity = pq.Quantity(new_value, new_units)
		self._q = original_quantity.simplified

		# Information to restore original representation.
		self.original_units = units
		self.original_multiplier = multiplier

		# Find the normalization factor.
		q, orig = self._q.magnitude, original_quantity.magnitude
		if q != orig:
			self.original_multiplier += log10(abs(q)) - log10(abs(orig))

	@property
	def dimensions(self):
		"""
		The set of simplified units and their exponents.
		"""

		return set(self._q.dimensionality.items())

	@property
	def value(self):
		"""
		The magnitude of the quantity, normalized to the base units.
		"""

		result = self._q.magnitude

		return result.tolist()

	@property
	def original_value(self):
		"""
		The magnitude of the quantity that matches the units.
		"""

		return self._q.magnitude / (10 ** self.original_multiplier)

	def assert_dimensions(self, other, exception=True):
		"""
		Whether the dimensions match.

		If exception is True and we would have returned False, raise an exception.
		"""

		if isinstance(other, basestring):
			# Given a units string.
			other = Quantity(1, other).dimensions
		elif isinstance(other, Quantity):
			# Given a Quantity.
			other = other.dimensions

		if self.dimensions == other:
			return True
		elif exception:
			raise IncompatibleDimensions(self.dimensions, other)
		else:
			return False

	# FIXME: Python 2.7 provides functools.total_ordering()
	def __eq__(self, other):
		try:
			self.assert_dimensions(other.dimensions)
		except AttributeError:
			raise TypeError('Expected dimensions for "{0!r}"'.format(other))

		return allclose(self.value, other.value)

	def __lt__(self, other):
		try:
			self.assert_dimensions(other.dimensions)
		except AttributeError:
			raise TypeError('Expected dimensions for "{0!r}"'.format(other))

		return self.value < other.value

	def __ne__(self, other):
		return not self == other

	def __le__(self, other):
		return self == other or self < other

	def __ge__(self, other):
		return not self < other

	def __gt__(self, other):
		return not self <= other

	def __abs__(self):
		if self.value < 0:
			return Quantity(abs(self.original_value), self.original_units)
		else:
			return self

	def __add__(self, other):
		"""
		Addition with matching dimensions.
		"""

		try:
			self.assert_dimensions(other.dimensions)
		except AttributeError:
			raise TypeError('Expected dimensions for "{0!r}"'.format(other))

		result = deepcopy(self)
		result._q += other._q

		return result

	def __sub__(self, other):
		"""
		Subtraction with matching dimensions.
		"""

		try:
			self.assert_dimensions(other.dimensions)
		except AttributeError:
			raise TypeError('Expected dimensions for "{0!r}"'.format(other))

		result = deepcopy(self)
		result._q -= other._q

		return result

	def __mul__(self, other):
		"""
		Multiplication by reals.
		"""

		result = deepcopy(self)
		result._q *= other

		return result

	def __rmul__(self, other):
		return self * other

	def __div__(self, other):
		"""
		Division by reals.
		"""

		result = deepcopy(self)
		result._q /= other

		return result

	def __repr__(self):
		return '{0}(\'{1}\')'.format(self.__class__.__name__, str(self))

	def __str__(self):
		value = self.original_value
		symbol = self.original_units

		return '{0:.10g} {1}'.format(value, symbol)

	def __deepcopy__(self, memo):
		"""
		Rather than copying anything, simply create a new instance.
		"""

		return Quantity(str(self))



from chaco.api import ArrayPlotData
from enable.api import Window
from functools import partial

from .common.chaco_plot import ChacoPlot

"""
An embeddable two-dimensional plot.
"""


class TwoDimensionalPlot(ChacoPlot):
	"""
	A 2D plot.
	"""

	auto_color_idx = 0
	auto_color_list = ['green', 'brown', 'blue', 'red', 'black']

	@classmethod
	def auto_color(cls):
		"""
		Choose the next color.
		"""

		color = cls.auto_color_list[cls.auto_color_idx]
		cls.auto_color_idx = (cls.auto_color_idx + 1) % len(cls.auto_color_list)

		return color

	def __init__(self, parent, color=None, *args, **kwargs):
		self.parent = parent

		if color is None:
			color = self.auto_color()

		self.data = ArrayPlotData()
		self.data.set_data('x', [0])
		self.data.set_data('y', [0])

		ChacoPlot.__init__(self, self.data, *args, **kwargs)

		self.plot(('x', 'y'), color=color)

		self.configure()

	@property
	def control(self):
		"""
		A drawable control.
		"""

		return Window(self.parent, component=self).control

	def get_data(self, axis):
		"""
		Values for an axis.
		"""

		return self.data.get_data(axis)

	def set_data(self, values, axis):
		self.data.set_data(axis, values)

	x_data = property(partial(get_data, axis='x'), partial(set_data, axis='x'))
	y_data = property(partial(get_data, axis='y'), partial(set_data, axis='y'))

	def x_autoscale(self):
		"""
		Enable autoscaling for the x axis.
		"""

		x_range = self.plots.values()[0][0].index_mapper.range
		x_range.low = x_range.high = 'auto'

	def y_autoscale(self):
		"""
		Enable autoscaling for the y axis.
		"""

		y_range = self.plots.values()[0][0].value_mapper.range
		y_range.low = y_range.high = 'auto'



from chaco.api import ArrayPlotData, ColorBar, HPlotContainer, jet, LinearMapper
from chaco.tools.api import RangeSelection, RangeSelectionOverlay
from enable.api import Window

from .common.chaco_plot import ChacoPlot

"""
An embeddable colormapped plot.
"""


class ColormappedPlot(ChacoPlot):
	"""
	A colormapped plot.
	"""

	def __init__(self, parent, x_bounds, y_bounds, *args, **kwargs):
		self.parent = parent

		self.data = ArrayPlotData()
		self.data.set_data('color', [[0]])

		ChacoPlot.__init__(self, self.data, *args, **kwargs)

		self.img_plot('color', colormap=jet, xbounds=x_bounds, ybounds=y_bounds)

		self.configure()

	@property
	def plot_obj(self):
		"""
		The actual plot object.
		"""

		return self.plots.values()[0][0]

	@property
	def control(self):
		"""
		A drawable control with a color bar.
		"""

		color_map = self.plot_obj.color_mapper
		linear_mapper = LinearMapper(range=color_map.range)
		color_bar = ColorBar(index_mapper=linear_mapper, color_mapper=color_map, plot=self.plot_obj,
				orientation='v', resizable='v', width=30)
		color_bar._axis.tick_label_formatter = self.sci_formatter
		color_bar.padding_top = self.padding_top
		color_bar.padding_bottom = self.padding_bottom
		color_bar.padding_left = 50 # Room for labels.
		color_bar.padding_right = 10

		range_selection = RangeSelection(component=color_bar)
		range_selection.listeners.append(self.plot_obj)
		color_bar.tools.append(range_selection)

		range_selection_overlay = RangeSelectionOverlay(component=color_bar)
		color_bar.overlays.append(range_selection_overlay)

		container = HPlotContainer(use_backbuffer=True)
		container.add(self)
		container.add(color_bar)

		return Window(self.parent, component=container).control

	@property
	def color_data(self):
		"""
		Plotted values.
		"""

		return self.data.get_data('color')

	@color_data.setter
	def color_data(self, values):
		self.data.set_data('color', values)

	@property
	def low_setting(self):
		"""
		Lowest color value.
		"""

		return self.plot_obj.color_mapper.range.low

	@low_setting.setter
	def low_setting(self, value):
		self.plot_obj.color_mapper.range.low_setting = value

	@property
	def high_setting(self):
		"""
		Highest color value.
		"""

		return self.plot_obj.color_mapper.range.high

	@high_setting.setter
	def high_setting(self, value):
		self.plot_obj.color_mapper.range.high_setting = value



from unittest import main

from ... import smf100a
from .. import mock_smf100a

from ...tests.server.test_smf100a import SMF100ATest


# Don't lose the real device.
real_SMF100A = smf100a.SMF100A
is_mock = SMF100ATest.mock


def setup():
	# Run the tests with a fake device.
	smf100a.SMF100A = mock_smf100a.MockSMF100A
	SMF100ATest.mock = True

def teardown():
	# Restore the real device for any remaining tests.
	smf100a.SMF100A = real_SMF100A
	SMF100ATest.mock = is_mock


if __name__ == '__main__':
	main()



import logging
log = logging.getLogger(__name__)

from spacq.interface.resources import Resource
from spacq.tool.box import Synchronized

from ..abstract_device import AbstractDevice
from ..tools import quantity_wrapped

"""
Agilent 34410A Digital Multimeter

Obtain measurements from the multimeter.
"""


class DM34410A(AbstractDevice):
	"""
	Interface for Agilent 34410A DM.

	Note: Currently supports only DC voltage readings.
	"""

	allowed_nplc = set([0.006, 0.02, 0.06, 0.2, 1.0, 2.0, 10.0, 100.0])
	allowed_auto_zero = set(['off', 'on', 'once'])

	def _setup(self):
		AbstractDevice._setup(self)

		# Resources.
		read_only = ['reading']
		for name in read_only:
			self.resources[name] = Resource(self, name)

		read_write = ['integration_time', 'auto_zero']
		for name in read_write:
			self.resources[name] = Resource(self, name, name)

		self.resources['reading'].units = 'V'
		self.resources['integration_time'].converter = float
		self.resources['integration_time'].allowed_values = self.allowed_nplc
		self.resources['auto_zero'].allowed_values = self.allowed_auto_zero

	@Synchronized()
	def _connected(self):
		AbstractDevice._connected(self)

		# TODO: Allow for measurements other than DC voltage.
		self.write('configure:voltage:dc')

	@Synchronized()
	def reset(self):
		"""
		Reset the device to its default state.
		"""

		log.info('Resetting "{0}".'.format(self.name))
		self.write('*rst')

	@property
	def integration_time(self):
		"""
		The integration time of the multimeter in terms of PLC.
		"""

		return float(self.ask('sense:voltage:dc:nplc?'))

	@integration_time.setter
	def integration_time(self, value):
		if value not in self.allowed_nplc:
			raise ValueError('Invalid NPLC value: {0}'.format(value))

		self.write('sense:voltage:dc:nplc {0}'.format(value))

	@property
	def auto_zero(self):
		"""
		The auto zero state.
		"""

		result = self.ask('sense:voltage:dc:zero:auto?')

		if result == '0':
			return 'off'
		elif result == '1':
			return 'on'

	@auto_zero.setter
	def auto_zero(self, value):
		if value not in self.allowed_auto_zero:
			raise ValueError('Invalid setting: {0}'.format(value))

		self.write('sense:voltage:dc:zero:auto {0}'.format(value))

	@property
	@quantity_wrapped('V')
	@Synchronized()
	def reading(self):
		"""
		The value measured by the device, as a quantity in V.
		"""

		self.status.append('Taking reading')

		try:
			log.debug('Getting reading.')
			result = float(self.ask('read?'))
			log.debug('Got reading: {0}'.format(result))

			return result
		finally:
			self.status.pop()


name = '34410A'
implementation = DM34410A



from unittest import main

from ... import voltage_source
from .. import mock_voltage_source

from ...tests.server.test_voltage_source import VoltageSourceTest


# Don't lose the real device.
real_VoltageSource = voltage_source.VoltageSource
is_mock = VoltageSourceTest.mock


def setup():
	# Run the tests with a fake device.
	voltage_source.VoltageSource = mock_voltage_source.MockVoltageSource
	VoltageSourceTest.mock = True

def teardown():
	# Restore the real device for any remaining tests.
	voltage_source.VoltageSource = real_VoltageSource
	VoltageSourceTest.mock = is_mock


if __name__ == '__main__':
	main()



import logging
log = logging.getLogger(__name__)

from unittest import main

from spacq.interface.units import Quantity
from spacq.tests.tool.box import DeviceServerTestCase

from ... import voltage_source


class VoltageSourceTest(DeviceServerTestCase):
	def obtain_device(self):
		return DeviceServerTestCase.obtain_device(self, impl=voltage_source.VoltageSource,
				manufacturer='IQC', model='Voltage source')

	def testCalibrate(self):
		"""
		Self-calibrate all the ports.
		"""

		vsrc = self.obtain_device()

		for port in vsrc.ports:
			port.apply_settings(calibrate=True)

	def testSetVoltages(self):
		"""
		Set voltages on all the ports.

		Note: Verification should also be done manually based on the voltage source output.
		"""

		vsrc = self.obtain_device()

		test_voltages = list(xrange(-10, 10 + 1, 2)) + list(xrange(5, 0, -1))

		for port, voltage in zip(xrange(16), test_voltages):
			vsrc.ports[port].voltage = Quantity(voltage, 'V')


if __name__ == '__main__':
	main()



import wx

from ....tool.box import MessageDialog
from ..two_dimensional import TwoDimensionalPlot
from .common.plot_setup import PlotSetupDialog


class TwoDimensionalPlotPanel(wx.Panel):
	def __init__(self, parent, x_data, y_data, x_label, y_label, *args, **kwargs):
		wx.Panel.__init__(self, parent, *args, **kwargs)

		# Panel.
		panel_box = wx.BoxSizer(wx.VERTICAL)

		## Plot.
		self.plot = TwoDimensionalPlot(self)
		panel_box.Add(self.plot.control, proportion=1, flag=wx.EXPAND)

		self.SetSizer(panel_box)

		self.plot.x_label, self.plot.y_label = x_label, y_label
		self.plot.x_data, self.plot.y_data = x_data, y_data


class TwoDimensionalPlotFrame(wx.Frame):
	def __init__(self, parent, x_data, y_data, x_label, y_label, *args, **kwargs):
		wx.Frame.__init__(self, parent, *args, **kwargs)

		# Frame.
		frame_box = wx.BoxSizer(wx.VERTICAL)

		## Plot panel.
		panel = TwoDimensionalPlotPanel(self, x_data, y_data, x_label, y_label)
		panel.SetMinSize((400, 300))
		frame_box.Add(panel, proportion=1, flag=wx.EXPAND)

		self.SetSizerAndFit(frame_box)


class TwoDimensionalPlotSetupDialog(PlotSetupDialog):
	def __init__(self, parent, headings, data, *args, **kwargs):
		PlotSetupDialog.__init__(self, parent, headings, ['x', 'y'],
				*args, **kwargs)

		self.parent = parent
		self.headings = headings
		self.data = data

	def make_plot(self):
		try:
			x_data, y_data = [self.data[:,axis].astype(float) for axis in self.axes]
		except ValueError as e:
			MessageDialog(self, str(e), 'Invalid value').Show()
			return

		x_label, y_label = [self.headings[x] for x in self.axes]
		title = '{0} vs {1}'.format(y_label, x_label)

		frame = TwoDimensionalPlotFrame(self.parent, x_data, y_data,
				x_label, y_label, title=title)
		frame.Show()

		return True



from nose.tools import eq_
from numbers import Real
from unittest import main

from spacq.tests.tool.box import DeviceServerTestCase

from ... import abc1234


class ABC1234Test(DeviceServerTestCase):
	def obtain_device(self):
		return DeviceServerTestCase.obtain_device(self, impl=abc1234.ABC1234,
				manufacturer='Sample', model='ABC1234')

	def testSetting(self):
		"""
		Test the setting.
		"""

		abc = self.obtain_device()
		abc.reset()

		eq_(abc.setting, 'default value')

		abc.setting = 'something else'
		eq_(abc.setting, 'something else')

		try:
			abc.setting = 'another thing'
		except ValueError:
			pass
		else:
			assert False, 'Expected ValueError.'

	def testGetValues(self):
		"""
		Obtain some values.
		"""

		abc = self.obtain_device()
		abc.reset()

		isinstance(abc.reading, Real)


if __name__ == '__main__':
	main()
